from unittest.mock import patch

from llama_index.core.base.llms.types import ChatResponse, ChatMessage, MessageRole
from llama_index.core.llms import MockLLM

from autorag.data.qa.query.llama_gen_query import (
	factoid_query_gen,
	concept_completion_query_gen,
	two_hop_incremental,
	custom_query_gen,
)
from autorag.data.qa.schema import QA
from tests.autorag.data.qa.query.base_test_query_gen import qa_df, multi_hop_qa_df

llm = MockLLM()
qa = QA(qa_df)


def test_make_factoid_query_gen():
	new_qa = qa.batch_apply(factoid_query_gen, llm=llm)
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(qa_df)


def test_make_factoid_query_gen_ko():
	new_qa = qa.batch_apply(factoid_query_gen, llm=llm, lang="ko")
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(qa_df)


def test_make_factoid_query_gen_ja():
	new_qa = qa.batch_apply(factoid_query_gen, llm=llm, lang="ja")
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(qa_df)


def test_concept_completion_query_gen():
	new_qa = qa.batch_apply(concept_completion_query_gen, llm=llm, lang="ko")
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(qa_df)


def test_concept_completion_query_gen_ja():
	new_qa = qa.batch_apply(concept_completion_query_gen, llm=llm, lang="ja")
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(qa_df)


def test_custom_query_gen():
	messages = [
		ChatMessage(
			role=MessageRole.SYSTEM,
			content="""As an expert AI assistant, create one question that can be directly answered from the provided **Text**.
Ensure question:

- Is clearly based on information within the **Text** and avoids assumptions or external context.
- Excludes any introductory phrases, such as "In the given text...", "Based on the context...", "Here are some questions baesd on context...".
- should include question sentence only.
- Is concise, relevant, and encourages specific, direct answers.
- has to be same as system language.
""",
		)
	]
	new_qa = qa.batch_apply(custom_query_gen, llm=llm, messages=messages)
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(qa_df)


async def two_hop_incremental_mock_achat(*args, **kwargs) -> ChatResponse:
	return ChatResponse(
		message=ChatMessage(
			role=MessageRole.ASSISTANT,
			content="""Answer: Tamaulipas
One-hop question (using Document 1): In which Mexican state is Nuevo Laredo located?
Two-hop question (using Document 2):  In which Mexican state can one find the Ciudad Deportiva, home to the Tecolotes de Nuevo Laredo?""",
		)
	)


@patch.object(MockLLM, "achat", two_hop_incremental_mock_achat)
def test_two_hop_incremental_query_gen():
	multi_hop_qa = QA(multi_hop_qa_df)
	new_qa = multi_hop_qa.batch_apply(two_hop_incremental, llm=llm, lang="ko")
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(multi_hop_qa_df)
	assert (
		new_qa.data["query"].iloc[0]
		== "In which Mexican state can one find the Ciudad Deportiva, home to the Tecolotes de Nuevo Laredo?"
	)



from datetime import datetime, date

import pytest

from autorag.nodes.passagefilter import RecencyFilter
from tests.autorag.nodes.passagefilter.test_passage_filter_base import (
	contents_example,
	time_list,
	ids_example,
	scores_example,
	base_passage_filter_test,
	base_passage_filter_node_test,
	previous_result,
	project_dir_with_corpus,
)


@pytest.fixture
def recency_filter_instance(project_dir_with_corpus):
	return RecencyFilter(project_dir=project_dir_with_corpus)


def test_recency_filter(recency_filter_instance):
	contents_result, id_result, score_result = recency_filter_instance._pure(
		contents_example,
		scores_example,
		ids_example,
		time_list,
		threshold_datetime=date(2021, 6, 30),
	)
	assert id_result[0] == [ids_example[0][1], ids_example[0][2], ids_example[0][3]]
	assert id_result[1] == [ids_example[1][2], ids_example[1][3]]
	assert contents_result[0] == [
		contents_example[0][1],
		contents_example[0][2],
		contents_example[0][3],
	]
	assert contents_result[1] == [contents_example[1][2], contents_example[1][3]]
	assert score_result[0] == [0.8, 0.1, 0.5]
	assert score_result[1] == [0.7, 0.3]
	base_passage_filter_test(contents_result, id_result, score_result)


def test_recency_filter_date_time_list(recency_filter_instance):
	time_list_2 = [
		[
			date(2015, 1, 1),
			datetime(2021, 9, 3),
			datetime(2022, 3, 5, 12, 30),
			datetime(2022, 3, 5, 12, 45, 00),
		],
		[
			date(2015, 1, 1),
			date(2021, 1, 1),
			datetime(2022, 3, 5, 12, 40),
			datetime(2022, 3, 5, 12, 45, 00),
		],
	]
	contents_result, id_result, score_result = recency_filter_instance._pure(
		contents_example,
		scores_example,
		ids_example,
		time_list_2,
		threshold_datetime=date(2021, 6, 30),
	)
	assert id_result[0] == [ids_example[0][1], ids_example[0][2], ids_example[0][3]]
	assert id_result[1] == [ids_example[1][2], ids_example[1][3]]
	assert contents_result[0] == [
		contents_example[0][1],
		contents_example[0][2],
		contents_example[0][3],
	]
	assert contents_result[1] == [contents_example[1][2], contents_example[1][3]]
	assert score_result[0] == [0.8, 0.1, 0.5]
	assert score_result[1] == [0.7, 0.3]
	base_passage_filter_test(contents_result, id_result, score_result)


def test_recency_filter_all_filtered(recency_filter_instance):
	contents_result, id_result, score_result = recency_filter_instance._pure(
		contents_example,
		scores_example,
		ids_example,
		time_list,
		threshold_datetime=datetime(2040, 6, 30),
	)
	assert id_result[0] == [ids_example[0][3]]
	assert id_result[1] == [ids_example[1][3]]
	assert contents_result[0] == [contents_example[0][3]]
	assert contents_result[1] == [contents_example[1][3]]
	assert score_result[0] == [0.5]
	assert score_result[1] == [0.3]
	base_passage_filter_test(contents_result, id_result, score_result)


def test_recency_filter_minutes(recency_filter_instance):
	contents_result, id_result, score_result = recency_filter_instance._pure(
		contents_example,
		scores_example,
		ids_example,
		time_list,
		threshold_datetime=datetime(2022, 3, 5, 12, 35),
	)
	assert id_result[0] == [ids_example[0][3]]
	assert id_result[1] == [ids_example[1][2], ids_example[1][3]]
	assert contents_result[0] == [contents_example[0][3]]
	assert contents_result[1] == [contents_example[1][2], contents_example[1][3]]
	assert score_result[0] == [0.5]
	assert score_result[1] == [0.7, 0.3]
	base_passage_filter_test(contents_result, id_result, score_result)


def test_recency_filter_seconds(recency_filter_instance):
	contents_result, id_result, score_result = recency_filter_instance._pure(
		contents_example,
		scores_example,
		ids_example,
		time_list,
		threshold_datetime=datetime(2022, 3, 5, 12, 40, 30),
	)
	assert id_result[0] == [ids_example[0][3]]
	assert id_result[1] == [ids_example[1][3]]
	assert contents_result[0] == [contents_example[0][3]]
	assert contents_result[1] == [contents_example[1][3]]
	assert score_result[0] == [0.5]
	assert score_result[1] == [0.3]
	base_passage_filter_test(contents_result, id_result, score_result)


def test_recency_filter_node(project_dir_with_corpus):
	result_df = RecencyFilter.run_evaluator(
		project_dir=project_dir_with_corpus,
		previous_result=previous_result,
		threshold_datetime=datetime(2021, 6, 30),
	)
	base_passage_filter_node_test(result_df)



import pandas as pd

from autorag.data.qa.schema import QA

passage = """NewJeans (뉴진스) is a 5-member girl group under ADOR and HYBE Labels.
The members consist of Minji, Hanni, Danielle, Haerin, and Hyein.
They released their debut single “Attention” on July 22, 2022,
followed by their debut extended play, New Jeans, which was released on August 1, 2022."""
question = "How many members are in the New Jeans?"
qa_df = pd.DataFrame(
	{
		"qid": ["jax"],
		"retrieval_gt": [[["havertz"]]],
		"retrieval_gt_contents": [[[passage]]],
		"query": [question],
	}
)


def check_generation_gt(result_qa: QA):
	assert isinstance(result_qa, QA)
	assert isinstance(result_qa.data, pd.DataFrame)
	assert set(result_qa.data.columns) == {
		"qid",
		"retrieval_gt",
		"retrieval_gt_contents",
		"query",
		"generation_gt",
	}
	assert len(result_qa.data) == len(qa_df)
	assert result_qa.data["generation_gt"].iloc[0]
	assert all(isinstance(x, list) for x in result_qa.data["generation_gt"].tolist())



import os
import pathlib
import tempfile

import pandas as pd
import pytest

from autorag.parser import Parser
from autorag.utils.util import load_summary_file

root_dir = pathlib.PurePath(os.path.dirname(os.path.realpath(__file__))).parent
resource_dir = os.path.join(root_dir, "resources")
result_dir = os.path.join(resource_dir, "test_results")
simple_data_glob = os.path.join(resource_dir, "parse_data", "all_files", "*")
full_data_glob = os.path.join(resource_dir, "parse_data", "all_files_full", "*")


@pytest.fixture
def simple_parser():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		simple_parser = Parser(data_path_glob=simple_data_glob, project_dir=project_dir)
		yield simple_parser


@pytest.fixture
def test_simple_parser():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		simple_parser = Parser(data_path_glob=simple_data_glob, project_dir=project_dir)
		yield simple_parser


@pytest.fixture
def full_parser():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		full_parser = Parser(data_path_glob=full_data_glob, project_dir=project_dir)
		yield full_parser


@pytest.fixture
def test_full_parser():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		full_parser = Parser(data_path_glob=full_data_glob, project_dir=project_dir)
		yield full_parser


def base_simple_yaml(parser, yaml_path):
	parser.start_parsing(yaml_path, all_files=False)
	project_dir = parser.project_dir
	assert os.path.exists(project_dir)
	assert os.path.exists(os.path.join(project_dir, "parse_config.yaml"))
	assert os.path.exists(os.path.join(project_dir, "pdf.parquet"))
	pdf_result = pd.read_parquet(os.path.join(project_dir, "pdf.parquet"))
	assert os.path.exists(os.path.join(project_dir, "csv.parquet"))
	assert os.path.exists(os.path.join(project_dir, "parsed_result.parquet"))
	parsed_result = pd.read_parquet(os.path.join(project_dir, "parsed_result.parquet"))

	expect_result_columns = ["texts", "path", "page", "last_modified_datetime"]
	assert all(
		[expect_column in pdf_result.columns for expect_column in expect_result_columns]
	)
	expect_result_columns = ["texts", "path", "page", "last_modified_datetime"]
	assert all(
		[
			expect_column in parsed_result.columns
			for expect_column in expect_result_columns
		]
	)
	summary_df = load_summary_file(os.path.join(project_dir, "summary.csv"))
	expect_summary_columns = [
		"filename",
		"module_name",
		"module_params",
		"execution_time",
	]
	assert all(
		[
			expect_column in summary_df.columns
			for expect_column in expect_summary_columns
		]
	)


def test_start_parsing_specific_type_with_perfect_simple_yaml(simple_parser):
	base_simple_yaml(
		simple_parser,
		os.path.join(resource_dir, "parse_data", "config", "perfect_simple_parse.yaml"),
	)


def test_start_parser_specific_type_with_lack_simple_yaml(simple_parser):
	base_simple_yaml(
		simple_parser,
		os.path.join(resource_dir, "parse_data", "config", "lack_simple_parse.yaml"),
	)


def base_full_yaml(parser, yaml_path):
	parser.start_parsing(yaml_path, all_files=False)
	project_dir = parser.project_dir
	assert os.path.exists(project_dir)
	assert os.path.exists(os.path.join(project_dir, "parse_config.yaml"))
	assert os.path.exists(os.path.join(project_dir, "pdf.parquet"))
	pdf_result = pd.read_parquet(os.path.join(project_dir, "pdf.parquet"))
	assert os.path.exists(os.path.join(project_dir, "csv.parquet"))
	assert os.path.exists(os.path.join(project_dir, "json.parquet"))
	assert os.path.exists(os.path.join(project_dir, "md.parquet"))
	assert os.path.exists(os.path.join(project_dir, "html.parquet"))
	assert os.path.exists(os.path.join(project_dir, "xml.parquet"))
	assert os.path.exists(os.path.join(project_dir, "parsed_result.parquet"))
	parsed_result = pd.read_parquet(os.path.join(project_dir, "parsed_result.parquet"))

	expect_result_columns = ["texts", "path", "page", "last_modified_datetime"]
	assert all(
		[expect_column in pdf_result.columns for expect_column in expect_result_columns]
	)
	expect_result_columns = ["texts", "path", "page", "last_modified_datetime"]
	assert all(
		[
			expect_column in parsed_result.columns
			for expect_column in expect_result_columns
		]
	)
	summary_df = load_summary_file(os.path.join(project_dir, "summary.csv"))
	expect_summary_columns = [
		"filename",
		"module_name",
		"module_params",
		"execution_time",
	]
	assert all(
		[
			expect_column in summary_df.columns
			for expect_column in expect_summary_columns
		]
	)


def test_start_parsing_specific_type_with_perfect_full_yaml(full_parser):
	base_full_yaml(
		full_parser,
		os.path.join(resource_dir, "parse_data", "config", "perfect_full_parse.yaml"),
	)


def test_start_parsing_specific_type_with_lack_full_yaml(full_parser):
	base_full_yaml(
		full_parser,
		os.path.join(resource_dir, "parse_data", "config", "lack_full_parse.yaml"),
	)


def test_start_parsing_all_files(simple_parser):
	simple_parser.start_parsing(
		os.path.join(resource_dir, "parse_data", "config", "all_files.yaml"),
		all_files=True,
	)
	project_dir = simple_parser.project_dir
	assert os.path.exists(project_dir)
	assert os.path.exists(os.path.join(project_dir, "parse_config.yaml"))
	assert os.path.exists(os.path.join(project_dir, "0.parquet"))
	all_files_result = pd.read_parquet(os.path.join(project_dir, "0.parquet"))

	expect_result_columns = ["texts", "path", "page", "last_modified_datetime"]
	assert all(
		[
			expect_column in all_files_result.columns
			for expect_column in expect_result_columns
		]
	)
	summary_df = load_summary_file(os.path.join(project_dir, "summary.csv"))
	expect_summary_columns = [
		"filename",
		"module_name",
		"module_params",
		"execution_time",
	]
	assert all(
		[
			expect_column in summary_df.columns
			for expect_column in expect_summary_columns
		]
	)



import os
import pickle
import shutil
import tempfile
from datetime import datetime

import pandas as pd
import pytest

from autorag.nodes.retrieval import BM25
from autorag.nodes.retrieval.bm25 import (
	bm25_ingest,
	tokenize_ko_kiwi,
	tokenize_porter_stemmer,
	tokenize_space,
	tokenize_ko_kkma,
	tokenize_ko_okt,
	tokenize_ja_sudachipy,
)
from autorag.utils.util import to_list
from tests.autorag.nodes.retrieval.test_retrieval_base import (
	queries,
	project_dir,
	corpus_df,
	previous_result,
	base_retrieval_test,
	base_retrieval_node_test,
	searchable_input_ids,
)

ko_texts = [
	"안녕? 나는 혜인이야. 내가 비눗방울 만드는 방법을 알려줄께.",
	"너 정말 잘한다. 넌 정말 짱이야. 우리 친구할래?",
	"내 생일 파티에 너만 못 온 그날, 혜진이가 엄청 혼났던 그날, 지원이가 여친이랑 헤어진 그날",
]

ja_texts = [
	"もう知っている あの日のことも あの頃のままで",
	"見つけられるよ この胸の奥底にある ずっと",
	"しょうがない もう少し待って ほら また笑えるから",
]


@pytest.fixture
def ingested_bm25_path():
	with tempfile.NamedTemporaryFile(suffix=".pkl", mode="w+b", delete=False) as path:
		bm25_ingest(path.name, corpus_df)
		yield path.name
		path.close()
		os.unlink(path.name)


@pytest.fixture
def bm25_instance(ingested_bm25_path):
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as temp_project_dir:
		os.makedirs(os.path.join(temp_project_dir, "resources"))
		os.makedirs(os.path.join(temp_project_dir, "data"))
		bm25_path = os.path.join(
			temp_project_dir, "resources", "bm25_porter_stemmer.pkl"
		)
		corpus_df.to_parquet(
			os.path.join(temp_project_dir, "data", "corpus.parquet"), index=False
		)
		shutil.copy(ingested_bm25_path, bm25_path)
		bm25 = BM25(project_dir=temp_project_dir)
		yield bm25


def test_bm25_retrieval(bm25_instance):
	top_k = 3
	id_result, score_result = bm25_instance._pure(queries, top_k=top_k)
	base_retrieval_test(id_result, score_result, top_k)


def test_bm25_retrieval_ids(bm25_instance):
	input_ids = [["doc2", "doc3"], ["doc1"], ["doc3", "doc4"]]
	id_result, score_result = bm25_instance._pure(queries, top_k=3, ids=input_ids)
	assert id_result == input_ids
	assert len(score_result) == 3
	assert len(score_result[0]) == 2
	assert len(score_result[1]) == 1
	assert len(score_result[2]) == 2


def test_bm25_retrieval_ids_empty(bm25_instance):
	input_ids = [["doc2", "doc3"], [], ["doc3"]]
	id_result, score_result = bm25_instance._pure(queries, top_k=3, ids=input_ids)
	assert id_result == input_ids
	assert len(score_result) == 3
	assert len(score_result[0]) == 2
	assert len(score_result[1]) == 0
	assert len(score_result[2]) == 1


def test_bm25_node():
	result_df = BM25.run_evaluator(
		project_dir=project_dir,
		previous_result=previous_result,
		top_k=4,
		bm25_tokenizer="gpt2",
	)
	base_retrieval_node_test(result_df)


def test_bm25_node_ids():
	result_df = BM25.run_evaluator(
		project_dir=project_dir,
		previous_result=previous_result,
		top_k=4,
		bm25_tokenizer="gpt2",
		ids=searchable_input_ids,
	)
	assert to_list(result_df["retrieved_ids"].tolist()) == searchable_input_ids
	score_result = to_list(result_df["retrieve_scores"].tolist())
	assert len(score_result) == 5
	assert len(score_result[0]) == 2


def test_bm25_ingest(ingested_bm25_path, bm25_instance):
	with open(ingested_bm25_path, "rb") as r:
		corpus = pickle.load(r)
	assert set(corpus.keys()) == {"tokens", "passage_id", "tokenizer_name"}
	assert isinstance(corpus["tokens"], list)
	assert isinstance(corpus["passage_id"], list)
	assert isinstance(corpus["tokenizer_name"], str)
	assert corpus["tokenizer_name"] == "porter_stemmer"
	assert len(corpus["tokens"]) == len(corpus["passage_id"]) == 5
	assert set(corpus["passage_id"]) == {"doc1", "doc2", "doc3", "doc4", "doc5"}

	top_k = 2
	id_result, score_result = bm25_instance._pure(
		[["What is test document?"], ["What is test document number 2?"]],
		top_k=top_k,
	)
	assert len(id_result) == len(score_result) == 2
	for id_list, score_list in zip(id_result, score_result):
		assert isinstance(id_list, list)
		assert isinstance(score_list, list)
		for _id in id_list:
			assert isinstance(_id, str)
			assert _id in ["doc1", "doc2", "doc3", "doc4", "doc5"]


def test_duplicate_id_bm25_ingest(ingested_bm25_path):
	new_doc_id = ["doc4", "doc5", "doc6", "doc7", "doc8"]
	new_contents = [
		"This is a test document 4.",
		"This is a test document 5.",
		"This is a test document 6.",
		"This is a test document 7.",
		"This is a test document 8.",
	]
	new_metadata = [{"datetime": datetime.now()} for _ in range(5)]
	new_corpus_df = pd.DataFrame(
		{"doc_id": new_doc_id, "contents": new_contents, "metadata": new_metadata}
	)
	bm25_ingest(ingested_bm25_path, new_corpus_df)
	with open(ingested_bm25_path, "rb") as r:
		corpus = pickle.load(r)
	assert len(corpus["tokens"]) == len(corpus["passage_id"]) == 8


def test_other_method_bm25():
	with pytest.raises(AssertionError):
		_ = BM25(project_dir=project_dir, bm25_tokenizer="space")


def test_tokenize_ko_wiki():
	tokenized_list = tokenize_ko_kiwi(ko_texts)
	assert len(tokenized_list) == len(ko_texts)
	assert isinstance(tokenized_list[0], list)
	assert all(isinstance(x, str) for x in tokenized_list[0])


def test_tokenize_ko_kkma():
	tokenized_list = tokenize_ko_kkma(ko_texts)
	assert len(tokenized_list) == len(ko_texts)
	assert isinstance(tokenized_list[0], list)
	assert all(isinstance(x, str) for x in tokenized_list[0])


def test_tokenize_ko_okt():
	tokenized_list = tokenize_ko_okt(ko_texts)
	assert len(tokenized_list) == len(ko_texts)
	assert isinstance(tokenized_list[0], list)
	assert all(isinstance(x, str) for x in tokenized_list[0])


def test_tokenize_porter_stemmer():
	texts = [
		"The best baseball team in the world is Kia Tigers.",
		"And for a fortnight there, we were forever. Run into you sometimes, ask about the weather",
		"I walked through the door with you. The air was cold.",
	]
	tokenized_list = tokenize_porter_stemmer(texts)
	assert len(tokenized_list) == len(texts)
	assert isinstance(tokenized_list[0], list)
	assert all(isinstance(x, str) for x in tokenized_list[0])


def test_tokenize_space():
	texts = [
		"내 생일 파티에 너만 못 온 그날, 혜진이가 엄청 혼났던 그날, 지원이가 여친이랑 헤어진 그날,,,, What's your ETA?  ",
		"  The best baseball team in the world is Kia Tigers. 최강 기아 타이거즈 최형우! 최형우!!! 기아의 해결사~   ",
		" You're my chemical hype boy (ah-ah) \n 내 지난날들은 눈 뜨면 잊는 꿈\nHype boy 너만 원해\nHype boy 내가 전해;",
	]
	tokenized_list = tokenize_space(texts)
	assert len(tokenized_list) == len(texts)
	assert isinstance(tokenized_list[0], list)
	assert all(isinstance(x, str) for x in tokenized_list[0])


def test_tokenize_sudachipy():
	tokenized_list = tokenize_ja_sudachipy(ja_texts)
	assert len(tokenized_list) == len(ja_texts)
	assert isinstance(tokenized_list[0], list)
	assert all(isinstance(x, str) for x in tokenized_list[0])



import pytest

from autorag.nodes.passagereranker.koreranker import KoReranker
from tests.autorag.nodes.passagereranker.test_passage_reranker_base import (
	base_reranker_test,
	base_reranker_node_test,
	ko_queries_example,
	ko_contents_example,
	ids_example,
	project_dir,
	ko_previous_result,
)
from tests.delete_tests import is_github_action


@pytest.fixture
def koreranker_instance():
	return KoReranker(project_dir)


@pytest.mark.skipif(is_github_action(), reason="Skipping this test on GitHub Actions")
def test_koreranker(koreranker_instance):
	top_k = 1
	contents_result, id_result, score_result = koreranker_instance._pure(
		ko_queries_example, ko_contents_example, ids_example, top_k
	)
	base_reranker_test(contents_result, id_result, score_result, top_k, use_ko=True)


@pytest.mark.skipif(is_github_action(), reason="Skipping this test on GitHub Actions")
def test_koreranker_batch_one(koreranker_instance):
	top_k = 1
	batch = 1
	contents_result, id_result, score_result = koreranker_instance._pure(
		ko_queries_example,
		ko_contents_example,
		ids_example,
		top_k,
		batch,
	)
	base_reranker_test(contents_result, id_result, score_result, top_k, use_ko=True)


@pytest.mark.skipif(is_github_action(), reason="Skipping this test on GitHub Actions")
def test_koreranker_node():
	top_k = 1
	result_df = KoReranker.run_evaluator(
		project_dir=project_dir, previous_result=ko_previous_result, top_k=top_k
	)
	base_reranker_node_test(result_df, top_k, use_ko=True)



import os
import pathlib
import tempfile
from datetime import datetime
from uuid import uuid4

import pandas as pd
import pytest

root_dir = pathlib.PurePath(
	os.path.dirname(os.path.realpath(__file__))
).parent.parent.parent
project_dir = os.path.join(root_dir, "resources", "sample_project")
qa_data = pd.read_parquet(os.path.join(project_dir, "data", "qa.parquet"))
corpus_data = pd.read_parquet(os.path.join(project_dir, "data", "corpus.parquet"))
previous_result = qa_data.sample(2)

queries_example = [
	"What is the capital of France?",
	"How many members are in Newjeans?",
]
contents_example = [
	[
		"NomaDamas is Great Team",
		"Paris is the capital of France.",
		"havertz is suck at soccer",
	],
	["i am hungry", "Newjeans has 5 members.", "LA is a country in the United States."],
]
ko_queries_example = ["프랑스의 수도는 어디인가요?", "뉴진스의 멤버는 몇 명인가요?"]
ko_contents_example = [
	[
		"마커AI는 멋진 회사입니다.",
		"프랑스의 수도는 파리 입니다.",
		"아스날은 축구를 못합니다.",
	],
	["배고파요", "뉴진스의 멤버는 5명 입니다.", "LA는 미국의 도시입니다."],
]
time_list = [
	[datetime(2021, 1, 1), datetime(2021, 1, 20), datetime(2021, 1, 3)],
	[datetime(2021, 3, 1), datetime(2030, 3, 1), datetime(1999, 1, 1)],
]
ids_example = [
	[str(uuid4()) for _ in range(len(contents_example[0]))],
	[str(uuid4()) for _ in range(len(contents_example[1]))],
]
scores_example = [[0.1, 0.8, 0.1], [0.1, 0.2, 0.7]]
f1_example = [0.4, 0.4]
recall_example = [1.0, 1.0]

previous_result["query"] = queries_example
previous_result["retrieved_contents"] = contents_example
previous_result["retrieved_ids"] = ids_example
previous_result["retrieve_scores"] = scores_example
previous_result["retrieval_f1"] = f1_example
previous_result["retrieval_recall"] = recall_example

ko_previous_result = previous_result.copy(deep=True)
ko_previous_result["query"] = ko_queries_example
ko_previous_result["retrieved_contents"] = ko_contents_example


def base_reranker_test(contents, ids, scores, top_k, use_ko=False, descending=True):
	assert len(contents) == len(ids) == len(scores) == 2
	assert len(contents[0]) == len(ids[0]) == len(scores[0]) == top_k
	for content_list, id_list, score_list in zip(contents, ids, scores):
		assert isinstance(content_list, list)
		assert isinstance(id_list, list)
		assert isinstance(score_list, list)
		for content, _id, score in zip(content_list, id_list, score_list):
			assert isinstance(content, str)
			assert isinstance(_id, str)
			assert isinstance(score, float)
		if descending is True:
			for i in range(1, len(score_list)):
				assert score_list[i - 1] >= score_list[i]
		else:
			for i in range(1, len(score_list)):
				assert score_list[i - 1] <= score_list[i]
	if use_ko is True:
		assert contents[0][0] == "프랑스의 수도는 파리 입니다."
		assert ids[0][0] in ids_example[0][1]
		assert contents[1][0] == "뉴진스의 멤버는 5명 입니다."
		assert ids[1][0] in ids_example[1][1]
	else:
		assert contents[0][0] == "Paris is the capital of France."
		assert ids[0][0] in ids_example[0][1]
		assert contents[1][0] == "Newjeans has 5 members."
		assert ids[1][0] in ids_example[1][1]


def base_reranker_node_test(result_df, top_k, use_ko=False, descending=True):
	contents = result_df["retrieved_contents"].tolist()
	ids = result_df["retrieved_ids"].tolist()
	scores = result_df["retrieve_scores"].tolist()
	base_reranker_test(contents, ids, scores, top_k, use_ko, descending=descending)


@pytest.fixture
def project_dir_with_corpus():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as temp_dir:
		data_dir = os.path.join(temp_dir, "data")
		os.makedirs(data_dir, exist_ok=True)
		qa_data.to_parquet(os.path.join(data_dir, "qa.parquet"), index=False)
		new_rows = pd.DataFrame(
			{
				"doc_id": ids_example[0] + ids_example[1],
				"contents": contents_example[0] + contents_example[1],
				"metadata": list(
					map(lambda x: {"last_modified_datetime": x}, time_list[0])
				)
				+ list(map(lambda x: {"last_modified_datetime": x}, time_list[1])),
			}
		)
		new_corpus = pd.concat([corpus_data, new_rows], ignore_index=True, axis=0)
		new_corpus.to_parquet(os.path.join(data_dir, "corpus.parquet"), index=False)
		yield temp_dir



import pandas as pd

prompt = "Answer this question: {query} \n\n {retrieved_contents}"
queries = ["What is the capital of Japan?", "What is the capital of China?"]
retrieved_contents = [
	[
		"Tokyo is the capital of Japan.",
		"Tokyo, the capital of Japan, is a huge metropolitan city.",
	],
	[
		"Beijing is the capital of China.",
		"Beijing, the capital of China, is a huge metropolitan city.",
	],
]
retrieve_scores = [[0.9, 0.8], [0.9, 0.8]]
retrieved_ids = [["doc1", "doc2"], ["doc3", "doc4"]]
previous_result = pd.DataFrame(
	{
		"query": queries,
		"retrieved_contents": retrieved_contents,
		"retrieve_scores": retrieve_scores,
		"retrieved_ids": retrieved_ids,
	}
)

doc_id = ["doc1", "doc2", "doc3", "doc4", "doc5"]
contents = [
	"This is a test document 1.",
	"This is a test document 2.",
	"This is a test document 3.",
	"This is a test document 4.",
	"This is a test document 5.",
]
metadata = [
	{"window": "havertz arsenal doosan minji naeun gaeun lets go"} for _ in range(5)
]
corpus_df = pd.DataFrame({"doc_id": doc_id, "contents": contents, "metadata": metadata})

retrieved_metadata = [
	[
		{"window": "havertz arsenal doosan minji naeun gaeun lets go"},
		{"window": "havertz arsenal doosan minji naeun gaeun lets go"},
	],
	[
		{"window": "havertz arsenal doosan minji naeun gaeun lets go"},
		{"window": "havertz arsenal doosan minji naeun gaeun lets go"},
	],
]



from autorag.data.parse import langchain_parse

from tests.autorag.data.parse.test_parse_base import (
	single_pdf_path_list,
	multiple_pdf_data_list,
	csv_data_list,
	json_data_list,
	markdown_data_list,
	html_data_list,
	xml_data_list,
	all_files_data_list,
	korean_text_glob,
	eng_text_glob,
	csv_glob,
	json_glob,
	markdown_glob,
	html_glob,
	xml_glob,
	all_files_glob,
	check_parse_result,
)


def test_langchain_parse_single_pdf():
	langchain_parse_original = langchain_parse.__wrapped__
	texts, path, pages = langchain_parse_original(
		single_pdf_path_list, parse_method="pdfminer"
	)
	check_parse_result(texts, path, "single_pdf")
	assert pages == [-1]


def test_langchain_parse_single_pdf_node():
	result_df = langchain_parse(
		korean_text_glob, file_type="pdf", parse_method="pdfminer"
	)
	check_parse_result(
		result_df["texts"].tolist(),
		result_df["path"].tolist(),
		"single_pdf",
	)
	assert result_df["page"].tolist() == [-1]


def test_langchain_parse_single_pdf_pages():
	langchain_parse_original = langchain_parse.__wrapped__
	texts, path, pages = langchain_parse_original(
		single_pdf_path_list, parse_method="pymupdf"
	)
	check_parse_result(texts, path, "single_pdf")
	assert pages == [1, 2]


def test_langchain_parse_single_pdf_pages_node():
	result_df = langchain_parse(
		korean_text_glob, file_type="pdf", parse_method="pymupdf"
	)
	check_parse_result(
		result_df["texts"].tolist(),
		result_df["path"].tolist(),
		"single_pdf",
	)
	assert result_df["page"].tolist() == [1, 2]


def test_langchain_parse_multiple_pdf():
	langchain_parse_original = langchain_parse.__wrapped__
	texts, path, pages = langchain_parse_original(
		multiple_pdf_data_list, parse_method="pdfminer"
	)
	check_parse_result(texts, path, "multiple_pdf")
	assert pages == [-1, -1]


def test_langchain_parse_multiple_pdf_node():
	result_df = langchain_parse(eng_text_glob, file_type="pdf", parse_method="pdfminer")
	check_parse_result(
		result_df["texts"].tolist(),
		result_df["path"].tolist(),
		"multiple_pdf",
	)
	assert result_df["page"].tolist() == [-1, -1]


def test_langchain_parse_multiple_pdf_pages():
	langchain_parse_original = langchain_parse.__wrapped__
	texts, path, pages = langchain_parse_original(
		multiple_pdf_data_list, parse_method="pymupdf"
	)
	check_parse_result(texts, path, "multiple_pdf")
	assert pages == [1, 1]


def test_langchain_parse_multiple_pdf_pages_node():
	result_df = langchain_parse(eng_text_glob, file_type="pdf", parse_method="pymupdf")
	check_parse_result(
		result_df["texts"].tolist(),
		result_df["path"].tolist(),
		"multiple_pdf",
	)
	assert result_df["page"].tolist() == [1, 1]


def test_langchain_csv():
	langchain_parse_original = langchain_parse.__wrapped__
	texts, path, pages = langchain_parse_original(csv_data_list, parse_method="csv")
	check_parse_result(texts, path, "csv")


def test_langchain_csv_node():
	result_df = langchain_parse(csv_glob, file_type="csv", parse_method="csv")
	check_parse_result(
		result_df["texts"].tolist(),
		result_df["path"].tolist(),
		"csv",
	)


def test_langchain_json():
	langchain_parse_original = langchain_parse.__wrapped__
	texts, path, pages = langchain_parse_original(
		json_data_list, parse_method="json", jq_schema=".content"
	)
	check_parse_result(texts, path, "json")
	assert texts == ["This is a sample JSON file"]


def test_langchain_json_node():
	result_df = langchain_parse(
		json_glob, file_type="json", parse_method="json", jq_schema=".content"
	)
	check_parse_result(
		result_df["texts"].tolist(),
		result_df["path"].tolist(),
		"json",
	)


def test_langchain_markdown():
	langchain_parse_original = langchain_parse.__wrapped__
	texts, path, pages = langchain_parse_original(
		markdown_data_list, parse_method="unstructuredmarkdown"
	)
	check_parse_result(texts, path, "markdown")


def test_langchain_markdown_node():
	result_df = langchain_parse(
		markdown_glob, file_type="md", parse_method="unstructuredmarkdown"
	)
	check_parse_result(
		result_df["texts"].tolist(),
		result_df["path"].tolist(),
		"markdown",
	)


def test_langchain_html():
	langchain_parse_original = langchain_parse.__wrapped__
	texts, path, pages = langchain_parse_original(html_data_list, parse_method="bshtml")
	check_parse_result(texts, path, "html")


def test_langchain_html_node():
	result_df = langchain_parse(html_glob, file_type="html", parse_method="bshtml")
	check_parse_result(
		result_df["texts"].tolist(),
		result_df["path"].tolist(),
		"html",
	)


def test_langchain_xml():
	langchain_parse_original = langchain_parse.__wrapped__
	texts, path, pages = langchain_parse_original(
		xml_data_list, parse_method="unstructuredxml"
	)
	check_parse_result(texts, path, "xml")


def test_langchain_xml_node():
	result_df = langchain_parse(
		xml_glob, file_type="xml", parse_method="unstructuredxml"
	)
	check_parse_result(
		result_df["texts"].tolist(),
		result_df["path"].tolist(),
		"xml",
	)


def test_langchain_all_files_unstructured():
	langchain_parse_original = langchain_parse.__wrapped__
	texts, path, pages = langchain_parse_original(
		all_files_data_list, parse_method="unstructured"
	)
	check_parse_result(texts, path, "all_files_unstructured")


def test_langchain_all_files_unstructured_node():
	result_df = langchain_parse(
		all_files_glob, file_type="all_files", parse_method="unstructured"
	)
	check_parse_result(
		result_df["texts"].tolist(),
		result_df["path"].tolist(),
		"all_files_unstructured",
	)


def test_langchain_all_files_directory():
	langchain_parse_original = langchain_parse.__wrapped__
	path_split_list = all_files_glob.split("/")
	glob_path = path_split_list.pop()
	folder_path = "/".join(path_split_list)
	texts, path, pages = langchain_parse_original(
		all_files_data_list, path=folder_path, glob=glob_path, parse_method="directory"
	)
	check_parse_result(texts, path, "all_files_directory")


def test_langchain_all_files_directory_node():
	result_df = langchain_parse(
		all_files_glob, file_type="all_files", parse_method="directory"
	)
	check_parse_result(
		result_df["texts"].tolist(), result_df["path"].tolist(), "all_files_directory"
	)



import time
from unittest.mock import patch

import openai.resources.beta.chat
from openai import AsyncOpenAI
from openai.types.chat import (
	ParsedChatCompletion,
	ParsedChatCompletionMessage,
	ParsedChoice,
)

from autorag.data.qa.query.openai_gen_query import (
	Response,
	factoid_query_gen,
	concept_completion_query_gen,
	two_hop_incremental,
	TwoHopIncrementalResponse,
)
from autorag.data.qa.schema import QA
from tests.autorag.data.qa.query.base_test_query_gen import qa_df, multi_hop_qa_df

client = AsyncOpenAI()


async def mock_gen_gt_response(*args, **kwargs) -> ParsedChatCompletion[Response]:
	return ParsedChatCompletion(
		id="test_id",
		choices=[
			ParsedChoice(
				finish_reason="stop",
				index=0,
				message=ParsedChatCompletionMessage(
					parsed=Response(query="mock answer"),
					role="assistant",
				),
			)
		],
		created=int(time.time()),
		model="gpt-4o-mini-2024-07-18",
		object="chat.completion",
	)


async def mock_two_hop_response(*args, **kwargs) -> ParsedChatCompletion[Response]:
	return ParsedChatCompletion(
		id="test_id",
		choices=[
			ParsedChoice(
				finish_reason="stop",
				index=0,
				message=ParsedChatCompletionMessage(
					parsed=TwoHopIncrementalResponse(
						answer="mock answer",
						one_hop_question="mock one hop question",
						two_hop_question="mock two hop question",
					),
					role="assistant",
				),
			)
		],
		created=int(time.time()),
		model="gpt-4o-mini-2024-07-18",
		object="chat.completion",
	)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_gen_gt_response,
)
def test_factoid_query_gen():
	qa = QA(qa_df)
	new_qa = qa.batch_apply(factoid_query_gen, client=client)
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(qa_df)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_gen_gt_response,
)
def test_factoid_query_gen_ko():
	qa = QA(qa_df)
	new_qa = qa.batch_apply(factoid_query_gen, client=client, lang="ko")
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(qa_df)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_gen_gt_response,
)
def test_factoid_query_gen_ja():
	qa = QA(qa_df)
	new_qa = qa.batch_apply(factoid_query_gen, client=client, lang="ja")
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(qa_df)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_gen_gt_response,
)
def test_concept_completion_query_gen_ko():
	qa = QA(qa_df)
	new_qa = qa.batch_apply(concept_completion_query_gen, client=client, lang="ko")
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(qa_df)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_gen_gt_response,
)
def test_concept_completion_query_gen_ja():
	qa = QA(qa_df)
	new_qa = qa.batch_apply(concept_completion_query_gen, client=client, lang="ja")
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(qa_df)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_two_hop_response,
)
def test_two_hop_incremental():
	qa = QA(multi_hop_qa_df)
	new_qa = qa.batch_apply(two_hop_incremental, client=client)
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(multi_hop_qa_df)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_two_hop_response,
)
def test_two_hop_incremental_ja():
	qa = QA(multi_hop_qa_df)
	new_qa = qa.batch_apply(two_hop_incremental, client=client, lang="ja")
	assert "query" in new_qa.data.columns
	assert all(isinstance(query, str) for query in new_qa.data["query"].tolist())
	assert len(new_qa.data) == len(multi_hop_qa_df)



import os
import shutil
import sys

import torch.cuda


def is_ubuntu() -> bool:
	try:
		with open("/etc/os-release") as f:
			for line in f:
				if line.startswith("NAME") and "Ubuntu" in line:
					return True
	except FileNotFoundError:
		pass
	return False


def is_github_action() -> bool:
	return is_ubuntu() and not torch.cuda.is_available()


def main():
	paths = sys.path
	for path in paths:
		tests_dir = os.path.join(path, "tests")
		if os.path.exists(tests_dir):
			print(f"Deleting {tests_dir}")
			shutil.rmtree(tests_dir)


if __name__ == "__main__":
	main()



import os
import pathlib
import tempfile
from datetime import datetime
from uuid import uuid4

import pandas as pd
import pytest

root_dir = pathlib.PurePath(
	os.path.dirname(os.path.realpath(__file__))
).parent.parent.parent
project_dir = os.path.join(root_dir, "resources", "sample_project")
qa_data = pd.read_parquet(os.path.join(project_dir, "data", "qa.parquet"))
corpus_data = pd.read_parquet(os.path.join(project_dir, "data", "corpus.parquet"))
previous_result = qa_data.sample(2)

queries_example = [
	"What is the capital of France?",
	"How many members are in Newjeans?",
]
contents_example = [
	[
		"NomaDamas is Great Team",
		"Paris is the capital of France.",
		"havertz is suck at soccer",
		"Paris is one of the capital from France. Isn't it?",
	],
	[
		"i am hungry",
		"LA is a country in the United States.",
		"Newjeans has 5 members.",
		"Danielle is one of the members of Newjeans.",
	],
]
time_list = [
	[
		datetime(2015, 1, 1),
		datetime(2021, 9, 3),
		datetime(2022, 3, 5, 12, 30),
		datetime(2022, 3, 5, 12, 45, 00),
	],
	[
		datetime(2015, 1, 1),
		datetime(2021, 1, 1),
		datetime(2022, 3, 5, 12, 40),
		datetime(2022, 3, 5, 12, 45, 00),
	],
]
ids_example = [
	[str(uuid4()) for _ in range(len(contents_example[0]))],
	[str(uuid4()) for _ in range(len(contents_example[1]))],
]
scores_example = [[0.1, 0.8, 0.1, 0.5], [0.1, 0.2, 0.7, 0.3]]
f1_example = [0.4, 1.0]
recall_example = [1.0, 0.3]

previous_result["query"] = queries_example
previous_result["retrieved_contents"] = contents_example
previous_result["retrieved_ids"] = ids_example
previous_result["retrieve_scores"] = scores_example
previous_result["retrieval_f1"] = f1_example
previous_result["retrieval_recall"] = recall_example


@pytest.fixture
def project_dir_with_corpus():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as temp_dir:
		data_dir = os.path.join(temp_dir, "data")
		os.makedirs(data_dir, exist_ok=True)
		qa_data.to_parquet(os.path.join(data_dir, "qa.parquet"), index=False)
		new_rows = pd.DataFrame(
			{
				"doc_id": ids_example[0] + ids_example[1],
				"contents": contents_example[0] + contents_example[1],
				"metadata": list(
					map(lambda x: {"last_modified_datetime": x}, time_list[0])
				)
				+ list(map(lambda x: {"last_modified_datetime": x}, time_list[1])),
			}
		)
		new_corpus = pd.concat([corpus_data, new_rows], ignore_index=True, axis=0)
		new_corpus.to_parquet(os.path.join(data_dir, "corpus.parquet"), index=False)
		yield temp_dir


def base_passage_filter_test(contents, ids, scores):
	assert len(contents) == len(ids) == len(scores) == 2
	for content_list, id_list, score_list in zip(contents, ids, scores):
		assert isinstance(content_list, list)
		assert isinstance(id_list, list)
		assert isinstance(score_list, list)
		for content, _id, score in zip(content_list, id_list, score_list):
			assert isinstance(content, str)
			assert isinstance(_id, str)
			assert isinstance(score, float)

	assert all([len(c) > 0 for c in contents])
	assert all([len(i) > 0 for i in ids])
	assert all([len(s) > 0 for s in scores])


def base_passage_filter_node_test(result_df):
	assert all(
		[
			column_name in result_df.columns
			for column_name in [
				"retrieved_contents",
				"retrieved_ids",
				"retrieve_scores",
			]
		]
	)
	base_passage_filter_test(
		result_df["retrieved_contents"].tolist(),
		result_df["retrieved_ids"].tolist(),
		result_df["retrieve_scores"].tolist(),
	)



import pytest

from autorag.nodes.promptmaker.fstring import Fstring
from tests.autorag.nodes.promptmaker.test_prompt_maker_base import (
	prompt,
	queries,
	retrieved_contents,
	previous_result,
)


@pytest.fixture
def fstring_instance():
	return Fstring("pseudo_project_dir")


def test_fstring(fstring_instance):
	result_prompts = fstring_instance._pure(prompt, queries, retrieved_contents)
	assert len(result_prompts) == 2
	assert isinstance(result_prompts, list)
	assert (
		result_prompts[0]
		== "Answer this question: What is the capital of Japan? \n\n Tokyo is the capital of Japan.\n\nTokyo, the capital of Japan, is a huge metropolitan city."
	)
	assert (
		result_prompts[1]
		== "Answer this question: What is the capital of China? \n\n Beijing is the capital of China.\n\nBeijing, the capital of China, is a huge metropolitan city."
	)


def test_fstring_node():
	result = Fstring.run_evaluator(
		project_dir="pseudo_project_dir", previous_result=previous_result, prompt=prompt
	)
	assert len(result) == 2
	assert result.columns == ["prompts"]
	assert (
		result["prompts"][0]
		== "Answer this question: What is the capital of Japan? \n\n Tokyo is the capital of Japan.\n\nTokyo, the capital of Japan, is a huge metropolitan city."
	)
	assert (
		result["prompts"][1]
		== "Answer this question: What is the capital of China? \n\n Beijing is the capital of China.\n\nBeijing, the capital of China, is a huge metropolitan city."
	)



import os
import pathlib
from datetime import datetime

from glob import glob

from autorag.data.parse.base import _add_last_modified_datetime
from autorag.data.utils.util import get_file_metadata

root_dir = pathlib.PurePath(
	os.path.dirname(os.path.realpath(__file__))
).parent.parent.parent
resource_dir = os.path.join(root_dir, "resources")
result_dir = os.path.join(resource_dir, "test_results")
data_dir = os.path.join(resource_dir, "parse_data")

korean_text_glob = os.path.join(data_dir, "korean_text", "*")
korean_table_glob = os.path.join(data_dir, "korean_table", "only_table", "*")
eng_text_glob = os.path.join(data_dir, "eng_text", "*")
csv_glob = os.path.join(data_dir, "csv_data", "*")
json_glob = os.path.join(data_dir, "json_data", "*")
markdown_glob = os.path.join(data_dir, "markdown_data", "*")
html_glob = os.path.join(data_dir, "html_data", "*")
xml_glob = os.path.join(data_dir, "xml_data", "*")
all_files_glob = os.path.join(data_dir, "all_files", "*")
hybrid_glob = os.path.join(data_dir, "hybrid_data", "*")

single_pdf_path_list = glob(korean_text_glob)
multiple_pdf_data_list = glob(eng_text_glob)
csv_data_list = glob(csv_glob)
json_data_list = glob(json_glob)
markdown_data_list = glob(markdown_glob)
html_data_list = glob(html_glob)
xml_data_list = glob(xml_glob)
all_files_data_list = glob(all_files_glob)
hybrid_data_list = glob(hybrid_glob)
table_data_list = glob(korean_table_glob)

file_path_dict = {
	"single_pdf": single_pdf_path_list,
	"multiple_pdf": multiple_pdf_data_list,
	"csv": csv_data_list,
	"json": json_data_list,
	"markdown": markdown_data_list,
	"html": html_data_list,
	"xml": xml_data_list,
	"all_files_unstructured": [
		all_files_data_list[0],
		all_files_data_list[1],
		all_files_data_list[1],
		all_files_data_list[1],
		all_files_data_list[1],
		all_files_data_list[1],
		all_files_data_list[1],
		all_files_data_list[1],
		all_files_data_list[1],
		all_files_data_list[1],
		all_files_data_list[1],
	],
	"all_files_directory": all_files_data_list,
	"hybrid": hybrid_data_list,
	"hybrid_text": multiple_pdf_data_list,
	"hybrid_table": table_data_list,
}


def check_parse_result(texts, file_names, file_type):
	assert isinstance(texts, list)
	assert isinstance(texts[0], str)
	assert all([file_name in file_path_dict[file_type] for file_name in file_names])


def test_last_modified_datetime():
	result = (
		[
			"jeffrey love bali, kia tigers and Newjeans. But it's a top secret that he loves Newjeans. i love this "
			"story."
		],
		single_pdf_path_list,
		[-1],
	)
	result = _add_last_modified_datetime(result)

	date = get_file_metadata(single_pdf_path_list[0])["last_modified_datetime"]
	assert result[3] == [date]



import os.path
import pathlib
import tempfile
from distutils.dir_util import copy_tree
from typing import Any
from unittest.mock import patch

import pandas as pd
import pytest
from llama_index.core.base.llms.types import CompletionResponse
from llama_index.embeddings.openai import OpenAIEmbedding
from llama_index.llms.openai import OpenAI

from autorag.deploy import extract_best_config
from autorag.evaluator import Evaluator
from autorag.nodes.retrieval import BM25, VectorDB, HybridRRF
from autorag.nodes.retrieval.run import run_retrieval_node
from autorag.schema import Node
from autorag.utils import validate_qa_dataset, validate_corpus_dataset
from autorag.utils.util import load_summary_file
from autorag.vectordb.milvus import Milvus
from tests.delete_tests import is_github_action
from tests.mock import mock_get_text_embedding_batch, mock_aget_text_embedding_batch

root_dir = pathlib.PurePath(os.path.dirname(os.path.realpath(__file__))).parent
resource_dir = os.path.join(root_dir, "resources")


@pytest.fixture
def evaluator():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		evaluator = Evaluator(
			os.path.join(resource_dir, "qa_data_sample.parquet"),
			os.path.join(resource_dir, "corpus_data_sample.parquet"),
			project_dir,
		)
		yield evaluator


@pytest.fixture
def test_evaluator():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		evaluator = Evaluator(
			os.path.join(resource_dir, "qa_test_data_sample.parquet"),
			os.path.join(resource_dir, "corpus_data_sample.parquet"),
			project_dir,
		)
		yield evaluator


def test_evaluator_init(evaluator):
	validate_qa_dataset(evaluator.qa_data)
	validate_corpus_dataset(evaluator.corpus_data)
	project_dir = evaluator.project_dir
	loaded_qa_data = pd.read_parquet(os.path.join(project_dir, "data", "qa.parquet"))
	loaded_corpus_data = pd.read_parquet(
		os.path.join(project_dir, "data", "corpus.parquet")
	)
	assert evaluator.qa_data.equals(loaded_qa_data)
	assert evaluator.corpus_data.equals(loaded_corpus_data)


def test_load_node_line(evaluator):
	node_lines = Evaluator._load_node_lines(os.path.join(resource_dir, "simple.yaml"))
	assert "retrieve_node_line" in list(node_lines.keys())
	assert node_lines["retrieve_node_line"] is not None
	nodes = node_lines["retrieve_node_line"]
	assert isinstance(nodes, list)
	assert len(nodes) == 3
	node = nodes[1]
	assert isinstance(node, Node)
	assert node.node_type == "retrieval"
	assert node.run_node == run_retrieval_node
	assert node.strategy["metrics"] == ["retrieval_f1", "retrieval_recall"]
	assert node.modules[0].module_type == "bm25"
	assert node.modules[1].module_type == "vectordb"
	assert node.modules[2].module_type == "hybrid_rrf"
	assert node.modules[0].module == BM25
	assert node.modules[1].module == VectorDB
	assert node.modules[2].module == HybridRRF
	assert node.modules[0].module_param == {
		"bm25_tokenizer": ["facebook/opt-125m", "porter_stemmer"]
	}
	assert node.modules[1].module_param == {
		"vectordb": ["openai_embed_3_large", "openai_embed_3_small"],
	}
	assert node.modules[2].module_param == {"weight_range": (4, 30)}
	assert nodes[2].node_type == "passage_filter"


@patch.object(
	OpenAIEmbedding,
	"get_text_embedding_batch",
	mock_get_text_embedding_batch,
)
@patch.object(
	OpenAIEmbedding,
	"aget_text_embedding_batch",
	mock_aget_text_embedding_batch,
)
def test_start_trial(evaluator):
	evaluator.start_trial(os.path.join(resource_dir, "simple.yaml"))
	project_dir = evaluator.project_dir
	assert os.path.exists(os.path.join(project_dir, "0"))
	assert os.path.exists(os.path.join(project_dir, "data"))
	assert os.path.exists(os.path.join(project_dir, "resources"))
	assert os.path.exists(os.path.join(project_dir, "trial.json"))
	assert os.path.exists(os.path.join(project_dir, "0", "config.yaml"))
	assert os.path.exists(os.path.join(project_dir, "0", "retrieve_node_line"))
	assert os.path.exists(
		os.path.join(project_dir, "0", "retrieve_node_line", "retrieval")
	)
	assert os.path.exists(
		os.path.join(project_dir, "0", "retrieve_node_line", "passage_filter")
	)
	assert os.path.exists(
		os.path.join(project_dir, "0", "retrieve_node_line", "retrieval", "0.parquet")
	)
	assert os.path.exists(
		os.path.join(project_dir, "0", "retrieve_node_line", "retrieval", "1.parquet")
	)
	expect_each_result_columns = [
		"retrieved_contents",
		"retrieved_ids",
		"retrieve_scores",
		"retrieval_f1",
		"retrieval_recall",
	]
	each_result_bm25 = pd.read_parquet(
		os.path.join(project_dir, "0", "retrieve_node_line", "retrieval", "0.parquet")
	)
	each_result_vectordb = pd.read_parquet(
		os.path.join(project_dir, "0", "retrieve_node_line", "retrieval", "1.parquet")
	)
	assert all(
		[
			expect_column in each_result_bm25.columns
			for expect_column in expect_each_result_columns
		]
	)
	assert all(
		[
			expect_column in each_result_vectordb.columns
			for expect_column in expect_each_result_columns
		]
	)
	expect_best_result_columns = [
		"qid",
		"query",
		"retrieval_gt",
		"generation_gt",
		"retrieved_contents",
		"retrieved_ids",
		"retrieve_scores",
		"retrieval_f1",
		"retrieval_recall",
	]
	summary_df = load_summary_file(
		os.path.join(project_dir, "0", "retrieve_node_line", "retrieval", "summary.csv")
	)
	best_row = summary_df.loc[summary_df["is_best"]].iloc[0]
	best_filename = summary_df.loc[summary_df["is_best"]]["filename"].values[0]
	best_result = pd.read_parquet(
		os.path.join(
			project_dir, "0", "retrieve_node_line", "retrieval", f"best_{best_filename}"
		)
	)
	assert all(
		[
			expect_column in best_result.columns
			for expect_column in expect_best_result_columns
		]
	)

	# test node line summary
	node_line_summary_path = os.path.join(
		project_dir, "0", "retrieve_node_line", "summary.csv"
	)
	assert os.path.exists(node_line_summary_path)
	node_line_summary_df = load_summary_file(
		node_line_summary_path, ["best_module_params"]
	)
	assert len(node_line_summary_df) == 3
	assert set(node_line_summary_df.columns) == {
		"node_type",
		"best_module_filename",
		"best_module_name",
		"best_module_params",
		"best_execution_time",
	}
	assert node_line_summary_df["node_type"][1] == "retrieval"
	assert node_line_summary_df["best_module_filename"][1] == best_row["filename"]
	assert node_line_summary_df["best_module_name"][1] == best_row["module_name"]
	assert node_line_summary_df["best_module_params"][1] == best_row["module_params"]
	assert node_line_summary_df["best_execution_time"][0] > 0

	# test trial summary
	trial_summary_path = os.path.join(project_dir, "0", "summary.csv")
	assert os.path.exists(trial_summary_path)
	trial_summary_df = load_summary_file(trial_summary_path, ["best_module_params"])
	assert len(trial_summary_df) == 3
	assert set(trial_summary_df.columns) == {
		"node_line_name",
		"node_type",
		"best_module_filename",
		"best_module_name",
		"best_module_params",
		"best_execution_time",
	}
	assert trial_summary_df["node_line_name"][0] == "retrieve_node_line"
	assert trial_summary_df["node_type"][1] == "retrieval"
	assert trial_summary_df["best_module_filename"][1] == best_row["filename"]
	assert trial_summary_df["best_module_name"][1] == best_row["module_name"]
	assert trial_summary_df["best_module_params"][1] == best_row["module_params"]
	assert trial_summary_df["best_execution_time"][0] > 0


@pytest.mark.skipif(
	is_github_action(),
	reason="This test needs milvus uri and token which is confidential.",
)
def test_start_trial_milvus(evaluator):
	milvus = Milvus(
		uri=os.environ["MILVUS_URI"],
		token=os.environ["MILVUS_TOKEN"],
		embedding_model="openai_embed_3_large",
		collection_name="openai_embed_3_large",
	)
	milvus.delete_collection()
	del milvus

	evaluator.start_trial(os.path.join(resource_dir, "simple_milvus.yaml"))
	project_dir = evaluator.project_dir
	assert os.path.exists(os.path.join(project_dir, "0"))
	assert os.path.exists(os.path.join(project_dir, "data"))
	assert os.path.exists(os.path.join(project_dir, "resources"))
	assert os.path.exists(os.path.join(project_dir, "trial.json"))
	assert os.path.exists(os.path.join(project_dir, "0", "config.yaml"))
	assert os.path.exists(os.path.join(project_dir, "0", "retrieve_node_line"))
	assert os.path.exists(
		os.path.join(project_dir, "0", "retrieve_node_line", "retrieval")
	)
	assert os.path.exists(
		os.path.join(project_dir, "0", "retrieve_node_line", "retrieval", "0.parquet")
	)
	assert os.path.exists(
		os.path.join(project_dir, "0", "retrieve_node_line", "retrieval", "1.parquet")
	)


@patch.object(
	OpenAIEmbedding,
	"get_text_embedding_batch",
	mock_get_text_embedding_batch,
)
@pytest.mark.skip(reason="This test is too slow")
def test_start_trial_full(evaluator):
	evaluator.start_trial(os.path.join(resource_dir, "full.yaml"))
	project_dir = evaluator.project_dir
	# full path check
	assert os.path.exists(os.path.join(project_dir, "0"))
	assert os.path.exists(os.path.join(project_dir, "data"))
	assert os.path.exists(os.path.join(project_dir, "resources"))
	assert os.path.exists(os.path.join(project_dir, "trial.json"))
	assert os.path.exists(os.path.join(project_dir, "0", "config.yaml"))

	# node line path check
	# 1. pre_retrieve_node_line
	assert os.path.exists(os.path.join(project_dir, "0", "pre_retrieve_node_line"))
	assert os.path.exists(
		os.path.join(project_dir, "0", "pre_retrieve_node_line", "query_expansion")
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "pre_retrieve_node_line", "query_expansion", "0.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "pre_retrieve_node_line", "query_expansion", "1.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "pre_retrieve_node_line", "query_expansion", "2.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "pre_retrieve_node_line", "query_expansion", "3.parquet"
		)
	)
	# 2. retrieve_node_line
	retrieval_node_line_path = os.path.join(project_dir, "0", "retrieve_node_line")
	assert os.path.exists(retrieval_node_line_path)
	retrieval_node_path = os.path.join(retrieval_node_line_path, "retrieval")
	assert os.path.exists(retrieval_node_path)
	assert all(
		[
			os.path.exists(os.path.join(retrieval_node_path, f"{i}.parquet"))
			for i in range(8)
		]
	)
	assert os.path.exists(
		os.path.join(project_dir, "0", "retrieve_node_line", "passage_reranker")
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "retrieve_node_line", "passage_reranker", "0.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "retrieve_node_line", "passage_reranker", "1.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "retrieve_node_line", "passage_reranker", "2.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(project_dir, "0", "retrieve_node_line", "passage_filter")
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "retrieve_node_line", "passage_filter", "0.parquet"
		)
	)
	# 3. post_retrieve_node_line
	assert os.path.exists(os.path.join(project_dir, "0", "post_retrieve_node_line"))
	assert os.path.exists(
		os.path.join(project_dir, "0", "post_retrieve_node_line", "prompt_maker")
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "post_retrieve_node_line", "prompt_maker", "0.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "post_retrieve_node_line", "prompt_maker", "1.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(project_dir, "0", "post_retrieve_node_line", "generator")
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "post_retrieve_node_line", "generator", "0.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "post_retrieve_node_line", "generator", "1.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "post_retrieve_node_line", "generator", "2.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "post_retrieve_node_line", "generator", "3.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "post_retrieve_node_line", "generator", "4.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "post_retrieve_node_line", "generator", "5.parquet"
		)
	)


async def mock_acomplete(self, prompt: str, formatted: bool = False, **kwargs: Any):
	return CompletionResponse(text=prompt)


async def mock_openai_apredict(self, prompt, *args, **kwargs):
	return "What is the capital of France?"


@patch.object(
	OpenAI,
	"acomplete",
	mock_acomplete,
)
@patch.object(
	OpenAI,
	"apredict",
	mock_openai_apredict,
)
@pytest.mark.skipif(is_github_action(), reason="Skipping this test on GitHub Actions")
def test_test_data_evaluate(test_evaluator):
	trial_folder = os.path.join(resource_dir, "result_project", "0")
	with tempfile.NamedTemporaryFile(
		mode="w+t", suffix=".yaml", delete=False
	) as yaml_file:
		extract_best_config(trial_folder, yaml_file.name)
		test_evaluator.start_trial(yaml_file.name, skip_validation=True)
		yaml_file.close()
		os.unlink(yaml_file.name)

	project_dir = test_evaluator.project_dir

	assert os.path.exists(os.path.join(project_dir, "0"))
	assert os.path.exists(os.path.join(project_dir, "data"))
	assert os.path.exists(os.path.join(project_dir, "resources"))
	assert os.path.exists(os.path.join(project_dir, "trial.json"))
	assert os.path.exists(os.path.join(project_dir, "0", "config.yaml"))
	assert os.path.exists(os.path.join(project_dir, "0", "retrieve_node_line"))
	assert os.path.exists(
		os.path.join(project_dir, "0", "retrieve_node_line", "retrieval")
	)
	assert os.path.exists(
		os.path.join(project_dir, "0", "retrieve_node_line", "retrieval", "0.parquet")
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "retrieve_node_line", "retrieval", "best_0.parquet"
		)
	)
	assert os.path.exists(
		os.path.join(project_dir, "0", "retrieve_node_line", "summary.csv")
	)
	assert os.path.exists(os.path.join(project_dir, "0", "pre_retrieve_node_line"))
	assert os.path.exists(
		os.path.join(project_dir, "0", "pre_retrieve_node_line", "query_expansion")
	)
	assert os.path.exists(
		os.path.join(
			project_dir, "0", "pre_retrieve_node_line", "query_expansion", "0.parquet"
		)
	)
	assert os.path.exists(os.path.join(project_dir, "0", "post_retrieve_node_line"))
	assert os.path.exists(
		os.path.join(project_dir, "0", "post_retrieve_node_line", "prompt_maker")
	)
	assert os.path.exists(
		os.path.join(project_dir, "0", "post_retrieve_node_line", "generator")
	)
	assert os.path.exists(os.path.join(project_dir, "0", "summary.csv"))


def base_restart_trial(evaluator, error_folder_path):
	error_path = os.path.join(evaluator.project_dir, "0")
	copy_tree(error_folder_path, error_path)
	evaluator.restart_trial(error_path)
	assert os.path.exists(os.path.join(error_path, "summary.csv"))
	assert os.path.exists(os.path.join(error_path, "pre_retrieve_node_line"))
	assert os.path.exists(
		os.path.join(error_path, "pre_retrieve_node_line", "summary.csv")
	)
	assert os.path.exists(os.path.join(error_path, "retrieve_node_line"))
	assert os.path.exists(os.path.join(error_path, "retrieve_node_line", "summary.csv"))
	assert os.path.exists(os.path.join(error_path, "post_retrieve_node_line"))
	assert os.path.exists(
		os.path.join(error_path, "post_retrieve_node_line", "summary.csv")
	)


async def mock_acomplete(self, messages, **kwargs):
	return CompletionResponse(text=messages)


async def mock_apredict(self, prompt, **kwargs):
	return prompt.format(**kwargs)


def test_restart_last_node(evaluator):
	compressor_error_folder_path = os.path.join(resource_dir, "result_project", "1")
	base_restart_trial(evaluator, compressor_error_folder_path)


@patch.object(OpenAI, "acomplete", mock_acomplete)
def test_restart_first_node(evaluator):
	prompt_error_folder_path = os.path.join(resource_dir, "result_project", "2")
	base_restart_trial(evaluator, prompt_error_folder_path)


def test_restart_leads_start_trial(evaluator):
	start_error_folder_path = os.path.join(resource_dir, "result_project")
	copy_tree(start_error_folder_path, evaluator.project_dir)
	error_path = os.path.join(evaluator.project_dir, "3")
	evaluator.restart_trial(error_path)
	restart_path = os.path.join(evaluator.project_dir, "4")
	assert os.path.exists(os.path.join(restart_path, "summary.csv"))
	assert os.path.exists(os.path.join(restart_path, "retrieve_node_line"))
	assert os.path.exists(
		os.path.join(restart_path, "retrieve_node_line", "summary.csv")
	)



import pytest

from autorag.data import chunk_modules, sentence_splitter_modules
from autorag.data.chunk import llama_index_chunk

from tests.autorag.data.chunk.test_chunk_base import (
	base_texts,
	parsed_result,
	check_chunk_result,
	check_chunk_result_node,
	base_metadata,
	expect_texts,
	expect_token_path,
	expect_token_idx,
	expect_overlap_idx,
)


@pytest.fixture
def chunk_instance():
	chunk_method = "token"
	kwargs = {"chunk_size": 50, "chunk_overlap": 0}
	chunk_instance = chunk_modules[chunk_method](**kwargs)
	yield chunk_instance


def test_llama_index_chunk(chunk_instance):
	llama_index_chunk_original = llama_index_chunk.__wrapped__
	doc_id, contents, path, start_end_idx, metadata = llama_index_chunk_original(
		base_texts, chunk_instance, metadata_list=base_metadata
	)
	check_chunk_result(doc_id, contents, path, start_end_idx, metadata)
	assert len(contents) == 4
	assert contents == expect_texts["token"]
	assert path == expect_token_path
	assert start_end_idx == expect_token_idx


def test_llama_index_chunk_node():
	result_df = llama_index_chunk(parsed_result, chunk_method="token")
	check_chunk_result_node(result_df)
	assert len(result_df["doc_id"].tolist()) == 9


def test_llama_index_chunk_file_name_ko(chunk_instance):
	llama_index_chunk_original = llama_index_chunk.__wrapped__
	doc_id, contents, path, start_end_idx, metadata = llama_index_chunk_original(
		base_texts,
		chunk_instance,
		file_name_language="ko",
		metadata_list=base_metadata,
	)
	check_chunk_result(doc_id, contents, path, start_end_idx, metadata)
	assert len(contents) == 4
	assert contents == expect_texts["token_ko"]
	assert path == expect_token_path
	assert start_end_idx == expect_token_idx


def test_llama_index_chunk_file_name_ko_node():
	result_df = llama_index_chunk(
		parsed_result, chunk_method="token", add_file_name="ko"
	)
	check_chunk_result_node(result_df)
	assert len(result_df["doc_id"].tolist()) == 9


def test_llama_index_chunk_file_name_eng(chunk_instance):
	llama_index_chunk_original = llama_index_chunk.__wrapped__
	doc_id, contents, path, start_end_idx, metadata = llama_index_chunk_original(
		base_texts,
		chunk_instance,
		file_name_language="en",
		metadata_list=base_metadata,
	)
	check_chunk_result(doc_id, contents, path, start_end_idx, metadata)
	assert len(contents) == 4
	assert contents == expect_texts["token_eng"]
	assert path == expect_token_path
	assert start_end_idx == expect_token_idx


def test_llama_index_chunk_file_name_eng_node():
	result_df = llama_index_chunk(
		parsed_result, chunk_method="token", add_file_name="en"
	)
	check_chunk_result_node(result_df)
	assert len(result_df["doc_id"].tolist()) == 9


def test_llama_index_chunk_file_name_ja(chunk_instance):
	llama_index_chunk_original = llama_index_chunk.__wrapped__
	doc_id, contents, path, start_end_idx, metadata = llama_index_chunk_original(
		base_texts,
		chunk_instance,
		file_name_language="ja",
		metadata_list=base_metadata,
	)
	check_chunk_result(doc_id, contents, path, start_end_idx, metadata)
	assert len(contents) == 4
	assert contents == expect_texts["token_ja"]
	assert path == expect_token_path
	assert start_end_idx == expect_token_idx


def test_llama_index_chunk_file_name_ja_node():
	result_df = llama_index_chunk(
		parsed_result, chunk_method="token", add_file_name="ja"
	)
	check_chunk_result_node(result_df)
	assert len(result_df["doc_id"].tolist()) == 9


@pytest.fixture
def chunk_instance_sentence_splitter():
	chunk_method = "sentencewindow"
	sentence_splitter_func = sentence_splitter_modules["kiwi"]()
	kwargs = {"sentence_splitter": sentence_splitter_func}
	chunk_instance_sentence_splitter = chunk_modules[chunk_method](**kwargs)
	yield chunk_instance_sentence_splitter


def test_llama_index_chunk_sentence(chunk_instance_sentence_splitter):
	llama_index_chunk_original = llama_index_chunk.__wrapped__
	doc_id, contents, path, start_end_idx, metadata = llama_index_chunk_original(
		base_texts, chunk_instance_sentence_splitter, metadata_list=base_metadata
	)
	check_chunk_result(doc_id, contents, path, start_end_idx, metadata)
	assert len(contents) == 2
	assert all("window" in meta.keys() for meta in metadata)
	assert path == [base_metadata[0]["path"], base_metadata[1]["path"]]
	assert start_end_idx == [(0, 168), (0, 165)]


def test_llama_index_chunk_sentence_node():
	result_df = llama_index_chunk(
		parsed_result, chunk_method="sentencewindow", sentence_splitter="kiwi"
	)
	check_chunk_result_node(result_df)
	assert len(result_df["doc_id"].tolist()) == 206
	assert all("window" in meta.keys() for meta in (result_df["metadata"].tolist()))


@pytest.fixture
def chunk_instance_overlap():
	chunk_method = "token"
	kwargs = {"chunk_size": 50, "chunk_overlap": 10}
	chunk_instance_overlap = chunk_modules[chunk_method](**kwargs)
	yield chunk_instance_overlap


def test_llama_index_chunk_overlap(chunk_instance_overlap):
	llama_index_chunk_original = llama_index_chunk.__wrapped__
	doc_id, contents, path, start_end_idx, metadata = llama_index_chunk_original(
		base_texts, chunk_instance_overlap, metadata_list=base_metadata
	)
	check_chunk_result(doc_id, contents, path, start_end_idx, metadata)
	assert len(contents) == 4
	assert contents == expect_texts["overlap"]
	assert path == expect_token_path
	assert start_end_idx == expect_overlap_idx



from pathlib import Path
from typing import Union

import pandas as pd

from autorag.schema import BaseModule


class TestModule(BaseModule):
	def __init__(self, project_dir: Union[str, Path], *args, **kwargs):
		self.param1 = kwargs.pop("param1", None)
		self.param2 = self.cast_to_init(project_dir, *args, **kwargs)

	def pure(self, previous_result: pd.DataFrame, *args, **kwargs):
		param3 = self.cast_to_run(previous_result, *args, **kwargs)
		return param3, self._pure(*args, **kwargs)

	def _pure(self, *args, **kwargs):
		return [
			kwargs.pop("param1", None),
			kwargs.pop("param2", None),
			kwargs.pop("param3", None),
			kwargs,
		]

	def cast_to_init(self, project_dir: Union[str, Path], *args, **kwargs):
		return kwargs.pop("param2", None)

	def cast_to_run(self, previous_result: pd.DataFrame, *args, **kwargs):
		return kwargs.pop("param3", None)


def test_base_module_kwargs_pop():
	param3, result_lst = TestModule.run_evaluator(
		"pseudo", previous_result=pd.DataFrame(), param1=1, param2=2, param3=3, param4=4
	)
	assert param3 == 3
	assert result_lst == [1, 2, 3, {"param4": 4}]



import os
import pathlib
import subprocess
import tempfile
from distutils.dir_util import copy_tree

from click.testing import CliRunner

from autorag.cli import cli

root_dir = pathlib.PurePath(os.path.dirname(os.path.realpath(__file__))).parent
resource_dir = os.path.join(root_dir, "resources")


def test_evaluator_cli():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		result = subprocess.run(
			[
				"autorag",
				"evaluate",
				"--config",
				os.path.join(resource_dir, "simple_mock.yaml"),
				"--qa_data_path",
				os.path.join(resource_dir, "qa_data_sample.parquet"),
				"--corpus_data_path",
				os.path.join(resource_dir, "corpus_data_sample.parquet"),
				"--project_dir",
				project_dir,
				"--skip_validation",
				"true",
			]
		)
		assert result.returncode == 0
		# check if the files are created
		assert os.path.exists(os.path.join(project_dir, "0"))
		assert os.path.exists(os.path.join(project_dir, "data"))
		assert os.path.exists(os.path.join(project_dir, "resources"))
		assert os.path.exists(os.path.join(project_dir, "trial.json"))
		assert os.path.exists(os.path.join(project_dir, "0", "retrieve_node_line"))
		assert os.path.exists(
			os.path.join(project_dir, "0", "retrieve_node_line", "retrieval")
		)
		assert os.path.exists(
			os.path.join(
				project_dir, "0", "retrieve_node_line", "retrieval", "0.parquet"
			)
		)


def test_validator_cli():
	result = subprocess.run(
		[
			"autorag",
			"validate",
			"--config",
			os.path.join(resource_dir, "simple_mock.yaml"),
			"--qa_data_path",
			os.path.join(resource_dir, "qa_data_sample.parquet"),
			"--corpus_data_path",
			os.path.join(resource_dir, "corpus_data_sample.parquet"),
		]
	)
	assert result.returncode == 0


def test_run_api():
	# This test code only tests cli function, not API server itself
	# If you are looking for API server test code, please go to test_deploy.py test_runner_api_server()
	runner = CliRunner()
	result = runner.invoke(
		cli,
		[
			"run_api",
			"--config_path",
			"test/path/test.yaml",
			"--host",
			"0.0.0.0",
			"--port",
			"8080",
		],
	)
	assert (
		result.exit_code == 1
	)  # it will occur error because I run this test with a wrong yaml path.
	# But it means that the command is working well. If not, it will occur exit_code 2.


def test_extract_best_config_cli():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		trial_path = os.path.join(resource_dir, "result_project", "0")
		output_path = os.path.join(project_dir, "best.yaml")
		subprocess.run(
			[
				"autorag",
				"extract_best_config",
				"--trial_path",
				trial_path,
				"--output_path",
				output_path,
			]
		)
		assert os.path.exists(output_path)


def test_restart_evaluate():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		original_path = os.path.join(resource_dir, "result_project")
		copy_tree(original_path, project_dir)
		trial_path = os.path.join(project_dir, "1")
		subprocess.run(["autorag", "restart_evaluate", "--trial_path", trial_path])
		assert os.path.exists(os.path.join(trial_path, "summary.csv"))


def test_restart_evaluate_leads_start_evaluate():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		original_path = os.path.join(resource_dir, "result_project")
		copy_tree(original_path, project_dir)
		trial_path = os.path.join(project_dir, "3")
		subprocess.run(["autorag", "restart_evaluate", "--trial_path", trial_path])
		restart_path = os.path.join(project_dir, "4")
		assert os.path.exists(os.path.join(restart_path, "summary.csv"))



import os.path
import pathlib
import tempfile

import pandas as pd
import pytest

from autorag import generator_models
from autorag.nodes.passagecompressor import TreeSummarize, Refine
from autorag.nodes.passagecompressor.run import run_passage_compressor_node
from autorag.utils.util import load_summary_file
from tests.mock import MockLLM

root_dir = pathlib.PurePath(
	os.path.dirname(os.path.realpath(__file__))
).parent.parent.parent
resources_dir = os.path.join(root_dir, "resources")
qa_data = pd.DataFrame(
	{
		"qid": ["id-1", "id-2", "id-3"],
		"query": ["query-1", "query-2", "query-3"],
		"retrieval_gt": [
			[["doc-1", "doc-2"]],
			[["doc-3"]],
			[["doc-5"], ["doc-6"]],
		],
		"generation_gt": [["generation-1"], ["generation-2"], ["generation-3"]],
	}
)
corpus_data = pd.DataFrame(
	{
		"doc_id": ["doc-1", "doc-2", "doc-3", "doc-4", "doc-5", "doc-6"],
		"contents": [
			"Enough for drinking water",
			"Just looking for a water bottle",
			"Do you want to buy some?",
			"I want to buy great octopus",
			"I want to buy some water",
			"Who is son? He is great player in the world",
		],
		"metadata": [
			{"datetime": "2021-01-01"},
			{"datetime": "2021-01-02"},
			{"datetime": "2021-01-03"},
			{"datetime": "2021-01-04"},
			{"datetime": "2021-01-05"},
			{"datetime": "2021-01-06"},
		],
	}
)
previous_result = pd.concat(
	[
		qa_data,
		pd.DataFrame(
			{
				"retrieved_ids": [
					["doc-1", "doc-3"],
					["doc-2", "doc-5"],
					["doc-5", "doc-6"],
				],
				"retrieved_contents": [
					["Enough for drinking water", "Do you want to buy some?"],
					["Just looking for a water bottle", "I want to buy some water"],
					[
						"I want to buy some water",
						"Who is son? He is great player in the world",
					],
				],
				"retrieve_scores": [
					[0.1, 0.2],
					[0.3, 0.4],
					[0.5, 0.6],
				],
			}
		),
	],
	axis=1,
)


@pytest.fixture
def node_line_dir():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		data_dir = os.path.join(project_dir, "data")
		os.makedirs(data_dir)
		qa_data.to_parquet(os.path.join(data_dir, "qa.parquet"), index=False)
		corpus_data.to_parquet(os.path.join(data_dir, "corpus.parquet"), index=False)
		trial_dir = os.path.join(project_dir, "trial_1")
		os.makedirs(trial_dir)
		node_line_dir = os.path.join(trial_dir, "node_line_1")
		os.makedirs(node_line_dir)
		yield node_line_dir


def test_run_passage_compressor_node(node_line_dir):
	generator_models["mock"] = MockLLM
	modules = [TreeSummarize, Refine]
	module_params = [
		{"llm": "mock", "model": "gpt-3.5-turbo-16k", "batch": 4},
		{"llm": "mock", "model": "gpt-3.5-turbo"},
	]
	strategies = {
		"metrics": ["retrieval_token_f1", "retrieval_token_precision"],
		"strategy": "normalize_mean",
		"speed_threshold": 5,
	}
	best_result = run_passage_compressor_node(
		modules, module_params, previous_result, node_line_dir, strategies
	)
	assert os.path.exists(os.path.join(node_line_dir, "passage_compressor"))
	assert set(best_result.columns) == {
		"qid",
		"query",
		"retrieval_gt",
		"generation_gt",
		"retrieved_contents",
		"retrieved_ids",
		"retrieve_scores",
		"passage_compressor_retrieval_token_f1",
		"passage_compressor_retrieval_token_precision",
	}
	# test summary feature
	summary_path = os.path.join(node_line_dir, "passage_compressor", "summary.csv")
	single_result_path = os.path.join(node_line_dir, "passage_compressor", "0.parquet")
	assert os.path.exists(single_result_path)
	single_result_df = pd.read_parquet(single_result_path)
	assert os.path.exists(summary_path)
	summary_df = load_summary_file(summary_path)
	assert set(summary_df.columns) == {
		"filename",
		"passage_compressor_retrieval_token_f1",
		"passage_compressor_retrieval_token_precision",
		"module_name",
		"module_params",
		"execution_time",
		"is_best",
	}
	assert len(summary_df) == 2
	assert summary_df["filename"][0] == "0.parquet"
	assert summary_df["passage_compressor_retrieval_token_f1"][0] == pytest.approx(
		single_result_df["retrieval_token_f1"].mean()
	)
	assert summary_df["passage_compressor_retrieval_token_precision"][
		0
	] == pytest.approx(single_result_df["retrieval_token_precision"].mean())
	assert summary_df["module_name"][0] == "TreeSummarize"
	assert summary_df["module_name"][1] == "Refine"
	assert summary_df["module_params"][0] == {
		"llm": "mock",
		"model": "gpt-3.5-turbo-16k",
		"batch": 4,
	}
	assert summary_df["execution_time"][0] > 0
	# test the best file is saved properly
	best_path = summary_df[summary_df["is_best"]]["filename"].values[0]
	assert os.path.exists(
		os.path.join(node_line_dir, "passage_compressor", f"best_{best_path}")
	)



import pytest
from autorag.vectordb.chroma import Chroma


@pytest.fixture
def chroma_ephemeral():
	return Chroma(
		embedding_model="mock",
		collection_name="test_collection",
		client_type="ephemeral",
	)


@pytest.mark.asyncio
async def test_add_and_query_documents(chroma_ephemeral):
	# Add documents
	ids = ["doc1", "doc2"]
	texts = ["This is a test document.", "This is another test document."]
	await chroma_ephemeral.add(ids, texts)

	# Query documents
	queries = ["test document"]
	contents, scores = await chroma_ephemeral.query(queries, top_k=2)

	assert len(contents) == 1
	assert len(scores) == 1
	assert len(contents[0]) == 2
	assert len(scores[0]) == 2
	assert scores[0][0] > scores[0][1]

	embeddings = await chroma_ephemeral.fetch([ids[0]])
	assert len(embeddings) == 1
	assert len(embeddings[0]) == 768

	exist = await chroma_ephemeral.is_exist([ids[0], "doc3"])
	assert len(exist) == 2
	assert exist[0] is True
	assert exist[1] is False


@pytest.mark.asyncio
async def test_delete_documents(chroma_ephemeral):
	# Add documents
	ids = ["doc1", "doc2"]
	texts = ["This is a test document.", "This is another test document."]
	await chroma_ephemeral.add(ids, texts)

	# Delete documents
	await chroma_ephemeral.delete([ids[0]])

	# Query documents to ensure they are deleted
	queries = ["test document"]
	contents, scores = await chroma_ephemeral.query(queries, top_k=2)

	assert len(contents[0]) == 1
	assert len(scores[0]) == 1



import time
from unittest.mock import patch

import openai.resources.beta.chat
from openai import AsyncOpenAI
from openai.types.chat import (
	ParsedChatCompletion,
	ParsedChoice,
	ParsedChatCompletionMessage,
)

from autorag.data.qa.generation_gt.openai_gen_gt import (
	make_concise_gen_gt,
	make_basic_gen_gt,
	Response,
)
from autorag.data.qa.schema import QA
from tests.autorag.data.qa.generation_gt.base_test_generation_gt import (
	qa_df,
	check_generation_gt,
)

client = AsyncOpenAI()


async def mock_gen_gt_response(*args, **kwargs) -> ParsedChatCompletion[Response]:
	return ParsedChatCompletion(
		id="test_id",
		choices=[
			ParsedChoice(
				finish_reason="stop",
				index=0,
				message=ParsedChatCompletionMessage(
					parsed=Response(answer="mock answer"),
					role="assistant",
				),
			)
		],
		created=int(time.time()),
		model="gpt-4o-mini-2024-07-18",
		object="chat.completion",
	)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_gen_gt_response,
)
def test_make_concise_gen_gt():
	qa = QA(qa_df)
	result_qa = qa.batch_apply(
		make_concise_gen_gt, client=client, model_name="gpt-4o-mini-2024-07-18"
	)
	check_generation_gt(result_qa)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_gen_gt_response,
)
def test_make_basic_gen_gt():
	qa = QA(qa_df)
	result_qa = qa.batch_apply(make_basic_gen_gt, client=client)
	check_generation_gt(result_qa)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_gen_gt_response,
)
def test_make_basic_gen_gt_ko():
	qa = QA(qa_df)
	result_qa = qa.batch_apply(make_basic_gen_gt, client=client, lang="ko")
	check_generation_gt(result_qa)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_gen_gt_response,
)
def test_make_basic_gen_gt_ja():
	qa = QA(qa_df)
	result_qa = qa.batch_apply(make_basic_gen_gt, client=client, lang="ja")
	check_generation_gt(result_qa)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_gen_gt_response,
)
def test_make_multiple_gen_gt():
	qa = QA(qa_df)
	result_qa = qa.batch_apply(make_basic_gen_gt, client=client, lang="ko").batch_apply(
		make_concise_gen_gt, client=client
	)
	check_generation_gt(result_qa)
	assert all(len(x) == 2 for x in result_qa.data["generation_gt"].tolist())



from autorag.nodes.queryexpansion import PassQueryExpansion
from tests.autorag.nodes.queryexpansion.test_query_expansion_base import (
	project_dir,
	previous_result,
)


def test_pass_query_expansion():
	result_df = PassQueryExpansion.run_evaluator(
		project_dir=project_dir, previous_result=previous_result
	)
	assert len(result_df) == 5
	queries = result_df["queries"].tolist()
	assert all(isinstance(query, list) for query in queries)
	assert len(queries[0]) == 1
	assert queries == list(map(lambda x: [x], previous_result["query"].tolist()))



import logging
import os
import pathlib
import uuid
from typing import Dict, Optional, List, Union, Literal

import pandas as pd
from pyngrok import ngrok
from quart import Quart, request, jsonify
from quart.helpers import stream_with_context
from pydantic import BaseModel, ValidationError

from autorag.deploy.base import BaseRunner
from autorag.nodes.generator.base import BaseGenerator
from autorag.nodes.promptmaker.base import BasePromptMaker
from autorag.utils import fetch_contents

logger = logging.getLogger("AutoRAG")

deploy_dir = pathlib.Path(__file__).parent
root_dir = pathlib.Path(__file__).parent.parent

VERSION_PATH = os.path.join(root_dir, "VERSION")


class QueryRequest(BaseModel):
	query: str
	result_column: Optional[str] = "generated_texts"


class RetrievedPassage(BaseModel):
	content: str
	doc_id: str
	filepath: Optional[str] = None
	file_page: Optional[int] = None
	start_idx: Optional[int] = None
	end_idx: Optional[int] = None


class RunResponse(BaseModel):
	result: Union[str, List[str]]
	retrieved_passage: List[RetrievedPassage]


class Passage(BaseModel):
	doc_id: str
	content: str
	score: float


class RetrievalResponse(BaseModel):
	passages: List[Passage]


class StreamResponse(BaseModel):
	"""
	When the type is generated_text, only generated_text is returned. The other fields are None.
	When the type is retrieved_passage, only retrieved_passage and passage_index are returned. The other fields are None.
	"""

	type: Literal["generated_text", "retrieved_passage"]
	generated_text: Optional[str]
	retrieved_passage: Optional[RetrievedPassage]
	passage_index: Optional[int]


class VersionResponse(BaseModel):
	version: str


class ApiRunner(BaseRunner):
	def __init__(self, config: Dict, project_dir: Optional[str] = None):
		super().__init__(config, project_dir)
		self.app = Quart(__name__)

		data_dir = os.path.join(project_dir, "data")
		self.corpus_df = pd.read_parquet(
			os.path.join(data_dir, "corpus.parquet"), engine="pyarrow"
		)
		self.__add_api_route()

	def __add_api_route(self):
		@self.app.route("/v1/run", methods=["POST"])
		async def run_query():
			try:
				data = await request.get_json()
				data = QueryRequest(**data)
			except ValidationError as e:
				return jsonify(e.errors()), 400

			previous_result = pd.DataFrame(
				{
					"qid": str(uuid.uuid4()),
					"query": [data.query],
					"retrieval_gt": [[]],
					"generation_gt": [""],
				}
			)  # pseudo qa data for execution
			for module_instance, module_param in zip(
				self.module_instances, self.module_params
			):
				new_result = module_instance.pure(
					previous_result=previous_result, **module_param
				)
				duplicated_columns = previous_result.columns.intersection(
					new_result.columns
				)
				drop_previous_result = previous_result.drop(columns=duplicated_columns)
				previous_result = pd.concat([drop_previous_result, new_result], axis=1)

			# Simulate processing the query
			generated_text = previous_result[data.result_column].tolist()[0]
			retrieved_passage = self.extract_retrieve_passage(previous_result)

			response = RunResponse(
				result=generated_text, retrieved_passage=retrieved_passage
			)

			return jsonify(response.model_dump()), 200

		@self.app.route("/v1/retrieve", methods=["POST"])
		async def run_retrieve_only():
			data = await request.get_json()
			query = data.get("query", None)
			if query is None:
				return jsonify(
					{
						"error": "Invalid request. You need to include 'query' in the request body."
					}
				), 400

			previous_result = pd.DataFrame(
				{
					"qid": str(uuid.uuid4()),
					"query": [query],
					"retrieval_gt": [[]],
					"generation_gt": [""],
				}
			)  # pseudo qa data for execution
			for module_instance, module_param in zip(
				self.module_instances, self.module_params
			):
				if isinstance(module_instance, BasePromptMaker) or isinstance(
					module_instance, BaseGenerator
				):
					continue
				new_result = module_instance.pure(
					previous_result=previous_result, **module_param
				)
				duplicated_columns = previous_result.columns.intersection(
					new_result.columns
				)
				drop_previous_result = previous_result.drop(columns=duplicated_columns)
				previous_result = pd.concat([drop_previous_result, new_result], axis=1)

			# Simulate processing the query
			retrieved_contents = previous_result["retrieved_contents"].tolist()[0]
			retrieved_ids = previous_result["retrieved_ids"].tolist()[0]
			retrieve_scores = previous_result["retrieve_scores"].tolist()[0]

			retrieval_response = RetrievalResponse(
				passages=[
					Passage(doc_id=doc_id, content=content, score=score)
					for doc_id, content, score in zip(
						retrieved_ids, retrieved_contents, retrieve_scores
					)
				]
			)
			return jsonify(retrieval_response.model_dump()), 200

		@self.app.route("/v1/stream", methods=["POST"])
		async def stream_query():
			try:
				data = await request.get_json()
				data = QueryRequest(**data)
			except ValidationError as e:
				return jsonify(e.errors()), 400

			@stream_with_context
			async def generate():
				previous_result = pd.DataFrame(
					{
						"qid": str(uuid.uuid4()),
						"query": [data.query],
						"retrieval_gt": [[]],
						"generation_gt": [""],
					}
				)  # pseudo qa data for execution

				for module_instance, module_param in zip(
					self.module_instances, self.module_params
				):
					if not isinstance(module_instance, BaseGenerator):
						new_result = module_instance.pure(
							previous_result=previous_result, **module_param
						)
						duplicated_columns = previous_result.columns.intersection(
							new_result.columns
						)
						drop_previous_result = previous_result.drop(
							columns=duplicated_columns
						)
						previous_result = pd.concat(
							[drop_previous_result, new_result], axis=1
						)
					else:
						retrieved_passages = self.extract_retrieve_passage(
							previous_result
						)
						for i, retrieved_passage in enumerate(retrieved_passages):
							yield (
								StreamResponse(
									type="retrieved_passage",
									generated_text=None,
									retrieved_passage=retrieved_passage,
									passage_index=i,
								)
								.model_dump_json()
								.encode("utf-8")
							)
						# Start streaming of the result
						assert len(previous_result) == 1
						prompt: str = previous_result["prompts"].tolist()[0]
						async for delta in module_instance.astream(
							prompt=prompt, **module_param
						):
							response = StreamResponse(
								type="generated_text",
								generated_text=delta,
								retrieved_passage=None,
								passage_index=None,
							)
							yield response.model_dump_json().encode("utf-8")

			return generate(), 200, {"X-Something": "value"}

		@self.app.route("/version", methods=["GET"])
		def get_version():
			with open(VERSION_PATH, "r") as f:
				version = f.read().strip()
			response = VersionResponse(version=version)
			return jsonify(response.model_dump()), 200

	def run_api_server(
		self, host: str = "0.0.0.0", port: int = 8000, remote: bool = True, **kwargs
	):
		"""
		Run the pipeline as an api server.
		Here is api endpoint documentation => https://docs.auto-rag.com/deploy/api_endpoint.html

		:param host: The host of the api server.
		:param port: The port of the api server.
		:param remote: Whether to expose the api server to the public internet using ngrok.
		:param kwargs: Other arguments for Flask app.run.
		"""
		logger.info(f"Run api server at {host}:{port}")
		if remote:
			http_tunnel = ngrok.connect(str(port), "http")
			public_url = http_tunnel.public_url
			logger.info(f"Public API URL: {public_url}")
		self.app.run(host=host, port=port, **kwargs)

	def extract_retrieve_passage(self, df: pd.DataFrame) -> List[RetrievedPassage]:
		retrieved_ids: List[str] = df["retrieved_ids"].tolist()[0]
		contents = fetch_contents(self.corpus_df, [retrieved_ids])[0]
		if "path" in self.corpus_df.columns:
			paths = fetch_contents(self.corpus_df, [retrieved_ids], column_name="path")[
				0
			]
		else:
			paths = [None] * len(retrieved_ids)
		metadatas = fetch_contents(
			self.corpus_df, [retrieved_ids], column_name="metadata"
		)[0]
		if "start_end_idx" in self.corpus_df.columns:
			start_end_indices = fetch_contents(
				self.corpus_df, [retrieved_ids], column_name="start_end_idx"
			)[0]
		else:
			start_end_indices = [None] * len(retrieved_ids)
		return list(
			map(
				lambda content, doc_id, path, metadata, start_end_idx: RetrievedPassage(
					content=content,
					doc_id=doc_id,
					filepath=path,
					file_page=metadata.get("page", None),
					start_idx=start_end_idx[0] if start_end_idx else None,
					end_idx=start_end_idx[1] if start_end_idx else None,
				),
				contents,
				retrieved_ids,
				paths,
				metadatas,
				start_end_indices,
			)
		)



import os
import pathlib

from autorag.validator import Validator

root_dir = pathlib.PurePath(os.path.dirname(os.path.realpath(__file__))).parent
resource_dir = os.path.join(root_dir, "resources")


def test_validator_validate():
	qa_path = os.path.join(resource_dir, "qa_data_sample.parquet")
	corpus_path = os.path.join(resource_dir, "corpus_data_sample.parquet")
	yaml_path = os.path.join(resource_dir, "simple_mock.yaml")

	validator = Validator(qa_path, corpus_path)
	validator.validate(yaml_path, qa_cnt=3, random_state=42)



import pandas as pd
import pytest

from autorag.nodes.retrieval import HybridCC
from autorag.nodes.retrieval.hybrid_cc import fuse_per_query, hybrid_cc
from tests.autorag.nodes.retrieval.test_hybrid_base import (
	sample_ids_2,
	sample_scores_2,
	sample_ids_3,
	sample_scores_3,
	base_hybrid_weights_node_test,
	sample_ids_non_overlap,
	pseudo_project_dir,
	previous_result,
)


def test_cc_fuse_per_query():
	result_id, result_scores = fuse_per_query(
		sample_ids_2[0],
		sample_ids_2[1],
		sample_scores_2[0],
		sample_scores_2[1],
		weight=0.3,
		top_k=3,
		normalize_method="mm",
		semantic_theoretical_min_value=-1.0,
		lexical_theoretical_min_value=0.0,
	)
	assert result_scores == pytest.approx([1.0, 0.23792, 0.175], rel=1e-3)
	assert result_id == ["id-1", "id-4", "id-2"]


def test_cc_non_overlap():
	result_id, result_scores = fuse_per_query(
		sample_ids_non_overlap[0],
		sample_ids_non_overlap[1],
		sample_scores_2[0],
		sample_scores_2[1],
		weight=0.3,
		top_k=3,
		normalize_method="mm",
		semantic_theoretical_min_value=-1.0,
		lexical_theoretical_min_value=0.0,
	)
	assert result_id == ["id-6", "id-1", "id-4"]
	assert result_scores == pytest.approx([0.7, 0.3, 0.2379237], rel=1e-3)


def test_hybrid_cc():
	result_id, result_scores = hybrid_cc(
		sample_ids_3, sample_scores_3, top_k=3, weight=0.0, normalize_method="tmm"
	)
	assert result_id[0] == ["id-1", "id-4", "id-3"]
	assert result_id[1] == ["id-2", "id-5", "id-4"]
	assert result_scores[0] == pytest.approx([1.0, 0.33333, 0.166666], rel=1e-3)
	assert result_scores[1] == pytest.approx([1.0, 0.4285714, 0.2857142], rel=1e-3)


def test_hybrid_cc_node(pseudo_project_dir):
	retrieve_scores = [1.0, 0.23792372, 0.175]
	base_hybrid_weights_node_test(
		HybridCC.run_evaluator, pseudo_project_dir, retrieve_scores
	)


def test_hybrid_cc_fixed_weight():
	result_id, result_scores = hybrid_cc(
		sample_ids_3, sample_scores_3, top_k=3, normalize_method="tmm", weight=0.4
	)
	assert result_id[0] == ["id-1", "id-4", "id-2"]
	assert result_id[1] == ["id-2", "id-5", "id-3"]
	assert result_scores[0] == pytest.approx([1.0, 0.2933333, 0.276666], rel=1e-3)
	assert result_scores[1] == pytest.approx([1.0, 0.39428, 0.38], rel=1e-3)
	assert isinstance(result_id, list)
	assert isinstance(result_scores, list)


def test_hybrid_cc_node_deploy(pseudo_project_dir):
	modules = {
		"target_modules": ("bm25", "vectordb"),
		"target_module_params": [
			{"top_k": 3},
			{"vectordb": "test_default", "top_k": 3},
		],
		"top_k": 3,
		"weight": 0.4,
	}
	hybrid_cc = HybridCC(project_dir=pseudo_project_dir, **modules)
	result_df = hybrid_cc.pure(previous_result=previous_result, **modules)
	assert len(result_df) == 3
	assert isinstance(result_df, pd.DataFrame)
	assert set(result_df.columns) == {
		"retrieved_contents",
		"retrieved_ids",
		"retrieve_scores",
	}
	assert len(result_df["retrieved_ids"].tolist()[0]) == 3
	assert len(result_df["retrieve_scores"].tolist()[0]) == 3
	assert len(result_df["retrieved_contents"].tolist()[0]) == 3



import os
import tempfile

import pytest
from langchain_core.documents import Document

from autorag.data.legacy.corpus import langchain_documents_to_parquet
from tests.autorag.data.legacy.corpus.test_base_corpus_legacy import validate_corpus


@pytest.fixture
def parquet_filepath():
	with tempfile.NamedTemporaryFile(suffix=".parquet", delete=False) as temp_file:
		yield temp_file.name
		temp_file.close()
		os.unlink(temp_file.name)


def test_langchain_documents_to_parquet(parquet_filepath):
	documents = [
		Document(page_content="test text", metadata={"key": "value"}) for _ in range(5)
	]
	result_df = langchain_documents_to_parquet(documents, parquet_filepath, upsert=True)
	validate_corpus(result_df, 5, parquet_filepath)



import os
import tempfile
from datetime import datetime

import pandas as pd
import pytest
import yaml

from autorag.nodes.retrieval.bm25 import bm25_ingest
from autorag.nodes.retrieval.vectordb import vectordb_ingest
from autorag.utils.util import get_event_loop
from autorag.vectordb.chroma import Chroma

sample_ids = (
	[["id-1", "id-2", "id-3"], ["id-2", "id-3", "id-4"]],
	[["id-1", "id-4", "id-3"], ["id-2", "id-5", "id-4"]],
)
sample_scores = ([[1, 3, 5], [2, 4, 6]], [[4, 2, 6], [5, 3, 7]])
sample_ids_2 = (
	["id-1", "id-2", "id-3", "id-4", "id-5"],
	["id-1", "id-4", "id-3", "id-5", "id-2"],
)
sample_scores_2 = ([5, 3, 1, 0.4, 0.2], [6, 2, 1, 0.5, 0.1])

sample_ids_3 = (
	[
		["id-1", "id-2", "id-3", "id-4", "id-5"],
		["id-2", "id-3", "id-4", "id-5", "id-6"],
	],
	[
		["id-1", "id-4", "id-3", "id-5", "id-2"],
		["id-2", "id-5", "id-4", "id-6", "id-3"],
	],
)
sample_scores_3 = (
	[[5, 3, 1, 0.4, 0.2], [6, 4, 2, 1.4, 1.2]],
	[
		[6, 2, 1, 0.5, 0.1],
		[7, 3, 2, 1.5, 1.1],
	],
)
sample_retrieval_gt_3 = [
	[["id-3"]],
	[["id-2"], ["id-6"]],
]

sample_ids_non_overlap = (
	["id-1", "id-2", "id-3", "id-4", "id-5"],
	["id-6", "id-4", "id-3", "id-7", "id-2"],
)

previous_result = pd.DataFrame(
	{
		"qid": ["query-1", "query-2", "query-3"],
		"query": ["query-1", "query-2", "query-3"],
		"retrieval_gt": [
			[["id-1"], ["id-2"], ["id-3"]],
			[["id-1"], ["id-2"], ["id-3"]],
			[["id-1"], ["id-2"], ["id-3"]],
		],
		"generation_gt": [["gen-1", "gen-2"], ["gen-1", "gen-2"], ["gen-1", "gen-2"]],
	}
)

modules_with_weights = {
	"ids": (
		[
			["id-1", "id-2", "id-3", "id-4", "id-5"],
			["id-1", "id-2", "id-3", "id-4", "id-5"],
		],
		[
			["id-1", "id-4", "id-3", "id-5", "id-2"],
			["id-1", "id-4", "id-3", "id-5", "id-2"],
		],
	),
	"scores": (
		[[5, 3, 1, 0.4, 0.2], [5, 3, 1, 0.4, 0.2]],
		[[6, 2, 1, 0.5, 0.1], [6, 2, 1, 0.5, 0.1]],
	),
	"top_k": 3,
	"weight": 0.3,
}


@pytest.fixture
def pseudo_project_dir():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		corpus_df = pd.DataFrame(
			{
				"doc_id": [
					"id-1",
					"id-2",
					"id-3",
					"id-4",
					"id-5",
					"id-6",
					"id-7",
					"id-8",
					"id-9",
				],
				"contents": [
					"doc-1",
					"doc-2",
					"doc-3",
					"doc-4",
					"doc-5",
					"doc-6",
					"doc-7",
					"doc-8",
					"doc-9",
				],
				"metadata": [{"last_modified_date": datetime.now()} for _ in range(9)],
			}
		)
		qa_df = pd.DataFrame(
			{
				"qid": ["havertz", "hanjunsu"],
				"query": ["What is JAX?", "Donggeon twerking jax havertz?"],
				"retrieval_gt": [
					[["id-1", "id-4"]],
					[["id-3", "id-8"], ["id-2"]],
				],
				"generation_gt": [
					["JAX is minsingjin."],
					["Donggeon is the god."],
				],
			}
		)
		os.makedirs(os.path.join(project_dir, "data"))
		corpus_df.to_parquet(os.path.join(project_dir, "data", "corpus.parquet"))
		qa_df.to_parquet(os.path.join(project_dir, "data", "qa.parquet"))
		resource_dir = os.path.join(project_dir, "resources")
		os.makedirs(resource_dir)
		bm25_ingest(os.path.join(resource_dir, "bm25_porter_stemmer.pkl"), corpus_df)
		chroma_path = os.path.join(resource_dir, "chroma")

		vectordb_config_path = os.path.join(resource_dir, "vectordb.yaml")
		with open(vectordb_config_path, "w") as f:
			vectordb_dict = {
				"vectordb": [
					{
						"name": "test_default",
						"db_type": "chroma",
						"embedding_model": "mock",
						"collection_name": "openai",
						"path": chroma_path,
						"similarity_metric": "cosine",
					}
				]
			}
			yaml.safe_dump(vectordb_dict, f)

		chroma = Chroma(
			embedding_model="mock",
			collection_name="openai",
			similarity_metric="cosine",
			client_type="persistent",
			path=chroma_path,
		)
		loop = get_event_loop()
		loop.run_until_complete(vectordb_ingest(chroma, corpus_df))
		yield project_dir


def base_hybrid_weights_node_test(hybrid_func, pseudo_project_dir, retrieve_scores):
	result = hybrid_func(
		project_dir=pseudo_project_dir,
		previous_result=previous_result,
		**modules_with_weights,
	)
	assert len(result) == 2
	assert isinstance(result, pd.DataFrame)
	assert set(result.columns) == {
		"retrieved_contents",
		"retrieved_ids",
		"retrieve_scores",
	}
	assert result["retrieved_ids"].tolist()[0] == ["id-1", "id-4", "id-2"]
	assert result["retrieve_scores"].tolist()[0] == pytest.approx(retrieve_scores)
	assert result["retrieved_contents"].tolist()[0] == ["doc-1", "doc-4", "doc-2"]



from autorag.nodes.passagefilter import PassPassageFilter
from tests.autorag.nodes.passagefilter.test_passage_filter_base import (
	project_dir,
	previous_result,
	contents_example,
	ids_example,
	scores_example,
)


def test_pass_passage_filter():
	result_df = PassPassageFilter.run_evaluator(
		project_dir=project_dir,
		previous_result=previous_result,
	)
	assert all(
		[
			column_name in result_df.columns
			for column_name in [
				"retrieved_contents",
				"retrieved_ids",
				"retrieve_scores",
			]
		]
	)
	assert result_df["retrieved_contents"].tolist() == contents_example
	assert result_df["retrieved_ids"].tolist() == ids_example
	assert result_df["retrieve_scores"].tolist() == scores_example



import time
from unittest.mock import patch

import openai.resources.chat
import pandas as pd
import pytest
from pydantic import BaseModel

from autorag.nodes.generator import OpenAILLM
from tests.autorag.nodes.generator.test_generator_base import (
	prompts,
	check_generated_texts,
	check_generated_tokens,
	check_generated_log_probs,
)
from tests.delete_tests import is_github_action
from tests.mock import mock_openai_chat_create
import openai.resources.beta.chat
from openai.types.chat import (
	ParsedChatCompletion,
	ParsedChatCompletionMessage,
	ParsedChoice,
)


@pytest.fixture
def openai_llm_instance():
	return OpenAILLM(
		project_dir=".", llm="gpt-3.5-turbo", api_key="mock_openai_api_key"
	)


@patch.object(
	openai.resources.chat.completions.AsyncCompletions,
	"create",
	mock_openai_chat_create,
)
def test_openai_llm(openai_llm_instance):
	answers, tokens, log_probs = openai_llm_instance._pure(
		prompts, temperature=0.5, logprobs=False, n=3
	)
	check_generated_texts(answers)
	check_generated_tokens(tokens)
	check_generated_log_probs(log_probs)


@patch.object(
	openai.resources.chat.completions.AsyncCompletions,
	"create",
	mock_openai_chat_create,
)
def test_openai_llm_node():
	previous_result = pd.DataFrame(
		{"prompts": prompts, "qid": ["id-1", "id-2", "id-3"]}
	)
	result_df = OpenAILLM.run_evaluator(
		project_dir=".",
		previous_result=previous_result,
		llm="gpt-4o-mini",
		api_key="mock_openai_api_key",
		temperature=0.5,
	)
	check_generated_texts(result_df["generated_texts"].tolist())
	check_generated_tokens(result_df["generated_tokens"].tolist())
	check_generated_log_probs(result_df["generated_log_probs"].tolist())


@patch.object(
	openai.resources.chat.completions.AsyncCompletions,
	"create",
	mock_openai_chat_create,
)
def test_openai_llm_truncate(openai_llm_instance):
	prompt = [
		f"havertz on the block and I am {i}th player on the Arsenal."
		for i in range(50_000)
	]
	prompt = " ".join(prompt)
	answers, tokens, log_probs = openai_llm_instance._pure([prompt] * 3)
	check_generated_texts(answers)
	check_generated_tokens(tokens)
	check_generated_log_probs(log_probs)


class TestResponse(BaseModel):
	name: str
	phone_number: str
	age: int
	is_dead: bool


async def mock_gen_gt_response(*args, **kwargs) -> ParsedChatCompletion[TestResponse]:
	return ParsedChatCompletion(
		id="test_id",
		choices=[
			ParsedChoice(
				finish_reason="stop",
				index=0,
				message=ParsedChatCompletionMessage(
					parsed=TestResponse(
						name="John Doe",
						phone_number="1234567890",
						age=30,
						is_dead=False,
					),
					role="assistant",
				),
			)
		],
		created=int(time.time()),
		model="gpt-4o-mini-2024-07-18",
		object="chat.completion",
	)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_gen_gt_response,
)
def test_openai_llm_structured():
	llm = OpenAILLM(project_dir=".", llm="gpt-4o-mini-2024-07-18")
	prompt = """You must transform the user introduction to json format. You have to extract four information: name, phone number, age, and is_dead.
Hello, my name is John Doe. My phone number is 1234567890. I am 30 years old. I am alive. I am good at soccer."""

	response = llm.structured_output([prompt], TestResponse)
	assert isinstance(response[0], TestResponse)
	assert response[0].name == "John Doe"
	assert response[0].phone_number == "1234567890"
	assert response[0].age == 30
	assert response[0].is_dead is False

	llm = OpenAILLM(project_dir=".", llm="gpt-3.5-turbo")
	with pytest.raises(ValueError):
		llm.structured_output([prompt], TestResponse)


@pytest.mark.skipif(
	is_github_action(),
	reason="Skipping this test on GitHub Actions because it uses the real OpenAI API.",
)
@pytest.mark.asyncio()
async def test_openai_llm_astream():
	import asyncstdlib as a

	llm_instance = OpenAILLM(project_dir=".", llm="gpt-4o-mini-2024-07-18")
	result = []
	async for i, s in a.enumerate(
		llm_instance.astream("Hello. Tell me about who is Kai Havertz")
	):
		assert isinstance(s, str)
		result.append(s)
		if i >= 1:
			assert len(result[i]) >= len(result[i - 1])



import asyncio
import os
import uuid

import pytest

from autorag.vectordb.qdrant import Qdrant
from tests.delete_tests import is_github_action


@pytest.mark.skipif(
	is_github_action(),
	reason="This test needs qdrant docker server.",
)
@pytest.fixture
def qdrant_instance():
	qdrant = Qdrant(
		embedding_model="mock",
		collection_name="autorag_t",
		client_type="docker",
		dimension=768,
	)
	yield qdrant
	qdrant.delete_collection()


@pytest.mark.skipif(
	is_github_action(),
	reason="This test needs qdrant docker server.",
)
@pytest.mark.asyncio
async def test_add_and_query_documents(qdrant_instance):
	# Add documents
	ids = [str(uuid.uuid4()) for _ in range(2)]
	texts = ["This is a test document.", "This is another test document."]
	await qdrant_instance.add(ids, texts)

	await asyncio.sleep(1)

	# Query documents
	queries = ["test document"]
	contents, scores = await qdrant_instance.query(queries, top_k=2)

	assert len(contents) == 1
	assert len(scores) == 1
	assert len(contents[0]) == 2
	assert len(scores[0]) == 2
	assert scores[0][0] > scores[0][1]

	embeddings = await qdrant_instance.fetch([ids[0]])
	assert len(embeddings) == 1
	assert len(embeddings[0]) == 768

	exist = await qdrant_instance.is_exist([ids[0], str(uuid.uuid4())])
	assert len(exist) == 2
	assert exist[0] is True
	assert exist[1] is False


@pytest.mark.skipif(
	is_github_action(),
	reason="This test needs qdrant docker server.",
)
@pytest.mark.asyncio
async def test_delete_documents(qdrant_instance):
	# Add documents
	ids = [str(uuid.uuid4()) for _ in range(2)]
	texts = ["This is a test document.", "This is another test document."]
	await qdrant_instance.add(ids, texts)

	await asyncio.sleep(1)

	# Delete documents
	await qdrant_instance.delete([ids[0]])

	# Query documents to ensure they are deleted
	queries = ["test document"]
	contents, scores = await qdrant_instance.query(queries, top_k=2)

	assert len(contents[0]) == 1
	assert len(scores[0]) == 1



import asyncio
import os
import pathlib
import shutil
import tempfile
import uuid
from datetime import datetime
from unittest.mock import patch

import pandas as pd
import pytest
import yaml
from llama_index.embeddings.openai import OpenAIEmbedding

from autorag.nodes.retrieval import VectorDB
from autorag.nodes.retrieval.vectordb import (
	vectordb_ingest,
	get_id_scores,
	filter_exist_ids_from_retrieval_gt,
	filter_exist_ids,
)
from autorag.vectordb.chroma import Chroma
from tests.autorag.nodes.retrieval.test_retrieval_base import (
	queries,
	corpus_df,
	previous_result,
	base_retrieval_test,
	base_retrieval_node_test,
	searchable_input_ids,
)
from tests.mock import mock_aget_text_embedding_batch

root_dir = pathlib.PurePath(
	os.path.dirname(os.path.realpath(__file__))
).parent.parent.parent
resource_path = os.path.join(root_dir, "resources")


@pytest.fixture
def mock_chroma():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as chroma_path:
		chroma = Chroma(
			client_type="persistent",
			path=chroma_path,
			embedding_model="mock",
			collection_name="test_vectordb_retrieval",
			similarity_metric="cosine",
		)
		yield chroma


@pytest.fixture
def openai_chroma():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as chroma_path:
		chroma = Chroma(
			client_type="persistent",
			path=chroma_path,
			embedding_model="openai",
			collection_name="test_vectordb_retrieval",
			similarity_metric="cosine",
		)
		yield chroma


@pytest.fixture
def project_dir_for_vectordb_node():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as test_project_dir:
		os.makedirs(os.path.join(test_project_dir, "resources"))
		chroma_path = os.path.join(test_project_dir, "resources", "chroma")
		os.makedirs(chroma_path)

		chroma_config = {
			"client_type": "persistent",
			"path": chroma_path,
			"embedding_model": "mock",
			"collection_name": "mock",
			"similarity_metric": "cosine",
		}
		vectordb_config_path = os.path.join(
			test_project_dir, "resources", "vectordb.yaml"
		)
		with open(vectordb_config_path, "w") as f:
			yaml.safe_dump(
				{"vectordb": [{"name": "mock", "db_type": "chroma", **chroma_config}]},
				f,
			)

		chroma = Chroma(
			**chroma_config,
		)
		os.makedirs(os.path.join(test_project_dir, "data"))
		corpus_path = os.path.join(test_project_dir, "data", "corpus.parquet")
		corpus_df.to_parquet(corpus_path, index=False)
		asyncio.run(vectordb_ingest(chroma, corpus_df))
		yield test_project_dir


@pytest.fixture
def project_dir_for_vectordb_node_from_sample_project():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as test_project_dir:
		sample_project_dir = os.path.join(resource_path, "sample_project")
		# copy & paste all folders and files in the sample_project folder
		shutil.copytree(sample_project_dir, test_project_dir, dirs_exist_ok=True)

		chroma_path = os.path.join(test_project_dir, "resources", "chroma")
		os.makedirs(chroma_path)
		chroma_config = {
			"client_type": "persistent",
			"path": chroma_path,
			"embedding_model": "mock",
			"collection_name": "mock",
			"similarity_metric": "cosine",
		}
		vectordb_config_path = os.path.join(
			test_project_dir, "resources", "vectordb.yaml"
		)
		with open(vectordb_config_path, "w") as f:
			yaml.safe_dump(
				{"vectordb": [{"name": "mock", "db_type": "chroma", **chroma_config}]},
				f,
			)

		chroma = Chroma(**chroma_config)
		corpus_path = os.path.join(test_project_dir, "data", "corpus.parquet")
		local_corpus_df = pd.read_parquet(corpus_path, engine="pyarrow")
		asyncio.run(vectordb_ingest(chroma, local_corpus_df))

		yield test_project_dir


@pytest.fixture
def vectordb_instance(project_dir_for_vectordb_node):
	vectordb = VectorDB(
		project_dir=project_dir_for_vectordb_node,
		vectordb="mock",
	)
	yield vectordb


def test_vectordb_retrieval(vectordb_instance):
	top_k = 4
	id_result, score_result = vectordb_instance._pure(
		queries,
		top_k=top_k,
	)
	base_retrieval_test(id_result, score_result, top_k)


def test_vectordb_retrieval_ids(vectordb_instance):
	ids = [["doc2", "doc3"], ["doc1", "doc2"], ["doc4", "doc5"]]
	id_result, score_result = vectordb_instance._pure(
		queries,
		top_k=4,
		ids=ids,
	)
	assert id_result == ids
	assert len(id_result) == len(score_result) == 3
	assert all([len(score_list) == 2 for score_list in score_result])


def test_vectordb_retrieval_ids_empty(vectordb_instance):
	ids = [["doc2", "doc3"], [], ["doc4"]]
	id_result, score_result = vectordb_instance._pure(
		queries,
		top_k=4,
		ids=ids,
	)
	assert id_result == ids
	assert len(id_result) == len(score_result) == 3
	assert len(score_result[0]) == 2
	assert len(score_result[1]) == 0
	assert len(score_result[2]) == 1


def test_vectordb_node(project_dir_for_vectordb_node_from_sample_project):
	result_df = VectorDB.run_evaluator(
		project_dir=project_dir_for_vectordb_node_from_sample_project,
		previous_result=previous_result,
		top_k=4,
		vectordb="mock",
	)
	base_retrieval_node_test(result_df)


def test_vectordb_node_ids(project_dir_for_vectordb_node_from_sample_project):
	result_df = VectorDB.run_evaluator(
		project_dir=project_dir_for_vectordb_node_from_sample_project,
		previous_result=previous_result,
		top_k=4,
		vectordb="mock",
		ids=searchable_input_ids,
	)
	contents = result_df["retrieved_contents"].tolist()
	ids = result_df["retrieved_ids"].tolist()
	scores = result_df["retrieve_scores"].tolist()
	assert len(contents) == len(ids) == len(scores) == 5
	assert len(contents[0]) == len(ids[0]) == len(scores[0]) == 2
	assert ids[0] == searchable_input_ids[0]


@patch.object(
	OpenAIEmbedding,
	"aget_text_embedding_batch",
	mock_aget_text_embedding_batch,
)
@pytest.mark.asyncio
async def test_duplicate_id_vectordb_ingest(openai_chroma):
	await vectordb_ingest(openai_chroma, corpus_df)
	assert openai_chroma.collection.count() == 5

	new_doc_id = ["doc4", "doc5", "doc6", "doc7", "doc8"]
	new_contents = [
		"This is a test document 4.",
		"This is a test document 5.",
		"This is a test document 6.",
		"This is a test document 7.",
		"This is a test document 8.",
	]
	new_metadata = [{"datetime": datetime.now()} for _ in range(5)]
	new_corpus_df = pd.DataFrame(
		{"doc_id": new_doc_id, "contents": new_contents, "metadata": new_metadata}
	)
	await vectordb_ingest(openai_chroma, new_corpus_df)

	assert openai_chroma.collection.count() == 8


@patch.object(
	OpenAIEmbedding,
	"aget_text_embedding_batch",
	mock_aget_text_embedding_batch,
)
@pytest.mark.asyncio
async def test_long_text_vectordb_ingest(openai_chroma):
	await vectordb_ingest(openai_chroma, corpus_df)
	new_doc_id = ["doc6", "doc7"]
	new_contents = ["This is a test" * 20000, "This is a test" * 40000]
	new_metadata = [{"datetime": datetime.now()} for _ in range(2)]
	new_corpus_df = pd.DataFrame(
		{"doc_id": new_doc_id, "contents": new_contents, "metadata": new_metadata}
	)
	await vectordb_ingest(openai_chroma, new_corpus_df)

	assert openai_chroma.collection.count() == 7


def mock_get_text_embedding_batch(self, texts, **kwargs):
	return [[3.0, 4.1, 3.2] for _ in range(len(texts))]


@patch.object(
	OpenAIEmbedding, "aget_text_embedding_batch", mock_aget_text_embedding_batch
)
@pytest.mark.asyncio
async def test_long_ids_ingest(openai_chroma):
	content_df = pd.DataFrame(
		{
			"doc_id": [str(uuid.uuid4()) for _ in range(10000)],
			"contents": ["havertz" for _ in range(10000)],
			"metadata": [
				{"last_modified_datetime": datetime.now()} for _ in range(10000)
			],
		}
	)
	await vectordb_ingest(openai_chroma, content_df)


@pytest.mark.asyncio
async def test_filter_exist_ids_from_retrieval_gt(mock_chroma):
	last_modified_datetime = datetime.now()
	ingested_df = pd.DataFrame(
		{
			"doc_id": ["id2"],
			"contents": ["content2"],
			"metadata": [{"last_modified_datetime": last_modified_datetime}],
		}
	)
	await vectordb_ingest(mock_chroma, ingested_df)

	# Create sample qa_data and corpus_data
	qa_data = pd.DataFrame(
		{
			"qid": ["qid1"],
			"query": ["query1"],
			"retrieval_gt": [[["id1", "id2"], ["id3"]]],
			"generation_gt": [["jaxjax"]],
		}
	)
	corpus_data = pd.DataFrame(
		{
			"doc_id": ["id1", "id2", "id3", "id4"],
			"contents": ["content1", "content2", "content3", "content4"],
			"metadata": [
				{"last_modified_datetime": last_modified_datetime} for _ in range(4)
			],
		}
	)

	# Call the function
	result = await filter_exist_ids_from_retrieval_gt(mock_chroma, qa_data, corpus_data)

	# Expected result
	expected_result = pd.DataFrame(
		{
			"doc_id": ["id1", "id3"],
			"contents": ["content1", "content3"],
			"metadata": [
				{
					"last_modified_datetime": last_modified_datetime,
					"prev_id": None,
					"next_id": None,
				}
				for _ in range(2)
			],
		}
	)

	# Assert the result
	pd.testing.assert_frame_equal(result.reset_index(drop=True), expected_result)


@pytest.mark.asyncio
async def test_filter_exist_ids(mock_chroma):
	last_modified_datetime = datetime.now()
	ingested_df = pd.DataFrame(
		{
			"doc_id": ["id2"],
			"contents": ["content2"],
			"metadata": [{"last_modified_datetime": last_modified_datetime}],
		}
	)
	await vectordb_ingest(mock_chroma, ingested_df)

	corpus_data = pd.DataFrame(
		{
			"doc_id": ["id1", "id2", "id3", "id4"],
			"contents": ["content1", "content2", "content3", "content4"],
			"metadata": [
				{"last_modified_datetime": last_modified_datetime} for _ in range(4)
			],
		}
	)

	result = await filter_exist_ids(mock_chroma, corpus_data)

	expected_result = pd.DataFrame(
		{
			"doc_id": ["id1", "id3", "id4"],
			"contents": ["content1", "content3", "content4"],
			"metadata": [
				{
					"last_modified_datetime": last_modified_datetime,
					"prev_id": None,
					"next_id": None,
				}
				for _ in range(3)
			],
		}
	)
	pd.testing.assert_frame_equal(result.reset_index(drop=True), expected_result)


def test_get_id_scores():
	query_embeddings = [[0.1, 0.2, 0.3], [0.4, 0.5, 0.6], [0.7, 0.8, 0.9]]
	content_embeddings = [[0.1, 0.2, 0.3], [0.4, 0.5, 0.6], [0.7, 0.8, 0.9]]
	similarity_metric = "cosine"

	scores = get_id_scores(query_embeddings, content_embeddings, similarity_metric)

	assert len(scores) == len(content_embeddings)
	assert all(isinstance(score, float) for score in scores)
	assert scores == pytest.approx([1.0, 1.0, 1.0])

	similarity_metric = "l2"
	scores = get_id_scores(query_embeddings, content_embeddings, similarity_metric)
	assert len(scores) == len(content_embeddings)
	assert all(isinstance(score, float) for score in scores)
	assert scores == pytest.approx(
		[1.0, 1.0, 1.0]
	)  # Assuming zero distance for identical vectors

	# Test for inner product
	similarity_metric = "ip"
	scores = get_id_scores(query_embeddings, content_embeddings, similarity_metric)
	assert len(scores) == len(content_embeddings)
	assert all(isinstance(score, float) for score in scores)
	assert scores == pytest.approx([0.5, 1.22, 1.94])



import os.path
import pathlib
import tempfile

from llama_index.core import MockEmbedding
from llama_index.embeddings.openai import OpenAIEmbedding

from autorag.vectordb import (
	load_vectordb,
	load_vectordb_from_yaml,
	load_all_vectordb_from_yaml,
)
from autorag.vectordb.chroma import Chroma


root_path = pathlib.PurePath(os.path.dirname(os.path.realpath(__file__))).parent.parent
resource_dir = os.path.join(root_path, "resources")


def test_load_vectordb():
	db = load_vectordb(
		"chroma",
		client_type="ephemeral",
		collection_name="jax1",
		embedding_model="mock",
	)
	assert isinstance(db, Chroma)
	assert db.collection.name == "jax1"


def test_load_vectordb_from_yaml():
	yaml_path = os.path.join(resource_dir, "simple_mock.yaml")
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		os.environ["PROJECT_DIR"] = project_dir
		default_vectordb = load_vectordb_from_yaml(yaml_path, "default", project_dir)
		assert isinstance(default_vectordb, Chroma)
		assert default_vectordb.collection.name == "openai"
		assert isinstance(default_vectordb.embedding, OpenAIEmbedding)

		chroma_default_vectordb = load_vectordb_from_yaml(
			yaml_path, "chroma_default", project_dir
		)
		assert isinstance(chroma_default_vectordb, Chroma)
		assert chroma_default_vectordb.collection.name == "openai"
		assert isinstance(chroma_default_vectordb.embedding, MockEmbedding)

		chroma_large_vectordb = load_vectordb_from_yaml(
			yaml_path, "chroma_large", project_dir
		)
		assert isinstance(chroma_large_vectordb, Chroma)
		assert chroma_large_vectordb.collection.name == "openai_embed_3_large"
		assert isinstance(chroma_large_vectordb.embedding, MockEmbedding)


def test_load_all_vectordb_from_yaml():
	yaml_path = os.path.join(resource_dir, "simple_mock.yaml")
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		os.environ["PROJECT_DIR"] = project_dir
		vectordb_list = load_all_vectordb_from_yaml(yaml_path, project_dir)
		assert len(vectordb_list) == 2

		chroma_default_vectordb = vectordb_list[0]
		assert isinstance(chroma_default_vectordb, Chroma)
		assert chroma_default_vectordb.collection.name == "openai"
		assert isinstance(chroma_default_vectordb.embedding, MockEmbedding)

		chroma_large_vectordb = vectordb_list[1]
		assert isinstance(chroma_large_vectordb, Chroma)
		assert chroma_large_vectordb.collection.name == "openai_embed_3_large"
		assert isinstance(chroma_large_vectordb.embedding, MockEmbedding)



import pandas as pd

from autorag.data.qa.sample import random_single_hop, range_single_hop
from autorag.data.qa.schema import Corpus, QA

corpus_df = pd.DataFrame(
	{"doc_id": [1, 2, 3, 4, 5], "contents": ["doc1", "doc2", "doc3", "doc4", "doc5"]}
)
corpus = Corpus(corpus_df)


def test_random_single_hop():
	qa = corpus.sample(random_single_hop, n=3)
	assert isinstance(qa, QA)
	assert len(qa.data) == 3
	assert set(qa.data.columns) == {"qid", "retrieval_gt"}
	assert qa.data["qid"].iloc[0]
	assert isinstance(qa.data["qid"].iloc[0], str)
	assert isinstance(qa.data["retrieval_gt"].iloc[0], list)
	assert qa.data["retrieval_gt"].iloc[0][0][0] in corpus_df["doc_id"].tolist()


def test_range_single_hop():
	qa = corpus.sample(range_single_hop, idx_range=range(3))
	assert isinstance(qa, QA)
	assert len(qa.data) == 3
	assert set(qa.data.columns) == {"qid", "retrieval_gt"}
	assert qa.data["qid"].iloc[0]
	assert isinstance(qa.data["qid"].iloc[0], str)
	assert isinstance(qa.data["retrieval_gt"].iloc[0], list)
	assert qa.data["retrieval_gt"].iloc[0][0][0] == 1



from autorag.support import get_support_modules, get_support_nodes


def test_get_support_modules():
	result = get_support_modules("bm25")
	assert result.__name__ == "BM25"


def test_get_support_nodes():
	result = get_support_nodes("retrieval")
	assert result.__name__ == "run_retrieval_node"



import pytest

from autorag.nodes.queryexpansion import HyDE
from tests.autorag.nodes.queryexpansion.test_query_expansion_base import (
	project_dir,
	previous_result,
	base_query_expansion_node_test,
)

sample_query = ["How many members are in Newjeans?", "What is visconde structure?"]


@pytest.fixture
def hyde_instance():
	return HyDE(
		project_dir=project_dir, generator_module_type="llama_index_llm", llm="mock"
	)


def test_hyde(hyde_instance):
	result = hyde_instance._pure(sample_query, prompt="", temperature=0.1)
	assert len(result[0]) == 1
	assert len(result) == 2


def test_hyde_node():
	generator_dict = {"generator_module_type": "llama_index_llm", "llm": "mock"}
	result_df = HyDE.run_evaluator(
		project_dir=project_dir,
		previous_result=previous_result,
		**generator_dict,
		temperature=0.1,
	)
	base_query_expansion_node_test(result_df)



import time
from unittest.mock import patch

import pandas as pd
from llama_index.core.base.llms.types import ChatResponse, ChatMessage, MessageRole
from llama_index.llms.openai import OpenAI
from openai import AsyncOpenAI
from openai.types.chat import (
	ParsedChatCompletion,
	ParsedChoice,
	ParsedChatCompletionMessage,
)
import openai.resources.beta.chat

from autorag.data.qa.filter.dontknow import (
	dontknow_filter_rule_based,
	dontknow_filter_openai,
	Response,
	dontknow_filter_llama_index,
)
from autorag.data.qa.schema import QA

en_qa_df = pd.DataFrame(
	{
		"generation_gt": [
			["I don't know what to say", "This is a test"],
			["This is another test", "I do not know the answer"],
			["This is fine", "All good"],
		]
	}
)

ko_qa_df = pd.DataFrame(
	{
		"generation_gt": [
			["몰라요", "테스트입니다"],
			["모르겠습니다", "이것은 테스트입니다"],
			["모르겠어요", "이것은 또 다른 테스트입니다"],
			["이것은 괜찮습니다", "모든 것이 좋습니다"],
		]
	}
)

ja_qa_df = pd.DataFrame(
	{
		"generation_gt": [
			["わかりません", "これはテストです"],
			["答えがわかりません", "これは別のテストです"],
			["これは大丈夫です", "すべて問題ありません"],
		]
	}
)

dont_know_lang = [
	"I don't know what to say",
	"I do not know the answer",
	"몰라요",
	"모르겠습니다",
	"모르겠어요",
	"わかりません",
	"答えがわかりません",
]

# Expected data after filtering
expected_df_en = pd.DataFrame({"generation_gt": [["This is fine", "All good"]]})

expected_df_ko = pd.DataFrame(
	{"generation_gt": [["이것은 괜찮습니다", "모든 것이 좋습니다"]]}
)
expected_df_ja = pd.DataFrame(
	{"generation_gt": [["これは大丈夫です", "すべて問題ありません"]]}
)


async def mock_openai_response(*args, **kwargs) -> ParsedChatCompletion[Response]:
	user_prompt = kwargs["messages"][1]["content"]
	return ParsedChatCompletion(
		id="test_id",
		choices=[
			ParsedChoice(
				finish_reason="stop",
				index=0,
				message=ParsedChatCompletionMessage(
					parsed=Response(is_dont_know=user_prompt in dont_know_lang),
					role="assistant",
				),
			)
		],
		created=int(time.time()),
		model="gpt-4o-mini-2024-07-18",
		object="chat.completion",
	)


async def mock_llama_index_response(*args, **kwargs) -> ChatResponse:
	user_prompt = kwargs["messages"][1].content
	return ChatResponse(
		message=ChatMessage(
			role=MessageRole.ASSISTANT, content=str(user_prompt in dont_know_lang)
		)
	)


def test_dontknow_filter_rule_based():
	# Test for English
	en_qa = QA(en_qa_df)
	result_en_qa = en_qa.filter(dontknow_filter_rule_based, lang="en").map(
		lambda df: df.reset_index(drop=True)
	)
	pd.testing.assert_frame_equal(result_en_qa.data, expected_df_en)

	# Test for Korean
	ko_qa = QA(ko_qa_df)
	result_ko_qa = ko_qa.filter(dontknow_filter_rule_based, lang="ko").map(
		lambda df: df.reset_index(drop=True)
	)
	pd.testing.assert_frame_equal(result_ko_qa.data, expected_df_ko)
	# Test for Japanese
	ja_qa = QA(ja_qa_df)
	result_ja_qa = ja_qa.filter(dontknow_filter_rule_based, lang="ja").map(
		lambda df: df.reset_index(drop=True)
	)
	pd.testing.assert_frame_equal(result_ja_qa.data, expected_df_ja)


@patch.object(
	openai.resources.beta.chat.completions.AsyncCompletions,
	"parse",
	mock_openai_response,
)
def test_dontknow_filter_openai():
	client = AsyncOpenAI()
	en_qa = QA(en_qa_df)
	result_en_qa = en_qa.batch_filter(
		dontknow_filter_openai, client=client, lang="en"
	).map(lambda df: df.reset_index(drop=True))
	pd.testing.assert_frame_equal(result_en_qa.data, expected_df_en)

	# Test for Korean
	ko_qa = QA(ko_qa_df)
	result_ko_qa = ko_qa.batch_filter(
		dontknow_filter_openai, client=client, lang="ko"
	).map(lambda df: df.reset_index(drop=True))
	pd.testing.assert_frame_equal(result_ko_qa.data, expected_df_ko)

	# Test for Japanese
	ja_qa = QA(ja_qa_df)
	result_ja_qa = ja_qa.batch_filter(
		dontknow_filter_openai, client=client, lang="ja"
	).map(lambda df: df.reset_index(drop=True))
	pd.testing.assert_frame_equal(result_ja_qa.data, expected_df_ja)


@patch.object(
	OpenAI,
	"achat",
	mock_llama_index_response,
)
def test_dontknow_filter_llama_index():
	llm = OpenAI()
	en_qa = QA(en_qa_df)
	result_en_qa = en_qa.batch_filter(
		dontknow_filter_llama_index, llm=llm, lang="en"
	).map(lambda df: df.reset_index(drop=True))
	pd.testing.assert_frame_equal(result_en_qa.data, expected_df_en)

	ko_qa = QA(ko_qa_df)
	result_ko_qa = ko_qa.batch_filter(
		dontknow_filter_llama_index, llm=llm, lang="ko"
	).map(lambda df: df.reset_index(drop=True))
	pd.testing.assert_frame_equal(result_ko_qa.data, expected_df_ko)

	ja_qa = QA(ja_qa_df)
	result_ja_qa = ja_qa.batch_filter(
		dontknow_filter_llama_index, llm=llm, lang="ja"
	).map(lambda df: df.reset_index(drop=True))
	pd.testing.assert_frame_equal(result_ja_qa.data, expected_df_ja)



import os
import pathlib
import tempfile

import pandas as pd
import pytest

from autorag.chunker import Chunker
from autorag.utils.util import load_summary_file

root_dir = pathlib.PurePath(os.path.dirname(os.path.realpath(__file__))).parent
resource_dir = os.path.join(root_dir, "resources")
data_dir = os.path.join(resource_dir, "chunk_data")
parsed_data_path = os.path.join(data_dir, "sample_parsed.parquet")


@pytest.fixture
def chunker():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		chunker = Chunker.from_parquet(
			parsed_data_path=parsed_data_path, project_dir=project_dir
		)
		yield chunker


@pytest.fixture
def chunker_test_fixture():
	with tempfile.TemporaryDirectory(ignore_cleanup_errors=True) as project_dir:
		chunker = Chunker.from_parquet(
			parsed_data_path=parsed_data_path, project_dir=project_dir
		)
		yield chunker


def test_start_chunking(chunker):
	chunker.start_chunking(os.path.join(resource_dir, "simple_chunk.yaml"))
	project_dir = chunker.project_dir
	assert os.path.exists(project_dir)
	assert os.path.exists(os.path.join(project_dir, "chunk_config.yaml"))
	assert os.path.exists(os.path.join(project_dir, "0.parquet"))
	chunk_result = pd.read_parquet(os.path.join(project_dir, "0.parquet"))
	expect_result_columns = ["doc_id", "contents", "metadata"]
	assert all(
		[
			expect_column in chunk_result.columns
			for expect_column in expect_result_columns
		]
	)
	summary_df = load_summary_file(os.path.join(project_dir, "summary.csv"))
	expect_summary_columns = [
		"filename",
		"module_name",
		"module_params",
		"execution_time",
	]
	assert all(
		[
			expect_column in summary_df.columns
			for expect_column in expect_summary_columns
		]
	)
